"""Light weight Python interface to HDF5"""

#  Copyright (C) 2010-2011     CSC - IT Center for Science Ltd.
#  Please see the accompanying LICENSE file for further information.

# The module is heavily inspired by h5py, however, no
# h5py code is used directly except where explicitly noted

import numpy as np
from _gpaw_hdf5 import *

def numpy_type_from_h5(datatype):
    """Simple conversion from HDF5 datatype to NumPy dtype"""
    cls = h5t_get_class(datatype)
    if cls == H5T_INTEGER:
        dtype = int
    elif cls == H5T_FLOAT:
        dtype = float
    elif cls == H5T_COMPOUND:
        dtype = complex
    elif cls == H5T_ENUM:
        dtype = bool
    elif cls == H5T_STRING:
        str_size = h5t_get_size(datatype)
        dtype = np.dtype('|S' + str(str_size))
    else:
        raise RuntimeError('Unsupported HDF5 datatype')

    return dtype

class Iterator:
    """Iterator over the datasets and subgroups in a File or in a Group."""

    def __init__(self, obj):
        self.idx = 0
        self.obj = obj
        self.nobj = obj.get_num_objs()

    def __iter__(self):
        return self

    def __next__(self):
        if self.idx == self.nobj:
            self.obj = None
            raise StopIteration

        key = self.obj.get_name_by_idx(self.idx)
        self.idx += 1
        return key

    next = __next__  # for Python 2
    
    
class Group:
    """This class defines a HDF5 group."""
    def __init__(self, loc_id, name, create=False):
        if create:
            self.id = h5g_create(loc_id, name)
        else:
            self.id = h5g_open(loc_id, name)

        self.attrs = Attributes(self.id)

        self.name = name
        self.opened = True

    def create_group(self, name):
        return Group(self.id, name, create=True)

    def create_dataset(self, name, shape, dtype):
        """Create a dataset with the NumPy equivalent type  and shape.

           Arguments:
           ----------
           dtype : Python or NumPy type
           shape : tuple containing the shape of dataset
           """

        return Dataset(self.id, name, dtype, shape, create=True)

    def get_num_objs(self):
        return h5g_get_num_objs(self.id)

    def get_name_by_idx(self, idx):
        return h5l_get_name_by_idx(self.id, idx)

    def close(self):
        h5g_close(self.id)
        self.opened = False

    # Dictionary like interface
    def keys(self):
        return list(self)

    def values(self):
        return [self[x] for x in self.keys()]

    def items(self):
        return [(x, self[x]) for x in self.keys()]

    def __getitem__(self, key):
        oid = h5o_open(self.id, key)
        obj_type = h5i_get_type(oid)
        h5o_close(oid)
        if obj_type == H5I_GROUP:
            return Group(self.id, key)
        elif obj_type == H5I_DATASET:
            return Dataset(self.id, key)
        else:
           # This shoul never happen
           raise RuntimeError('Accessing unknown object type')

    def __iter__(self):
        return Iterator(self)

    def __len__(self):
        return self.get_num_objs()

    def __del__(self):
        if self.opened:
            self.close()

class File(Group):
    """This class defines a HDF5 file."""

    def __init__(self, name, mode='r', comm=None):
        """If a communicator is passed as an argument, file is
           created or opened for parallel IO."""

        if comm is None:
            plist = H5P_DEFAULT
        else:
            plist = h5p_create(H5P_FILE_ACCESS)
            h5p_set_fapl_mpio(plist, comm)

        if mode == 'r':
            self.id = h5f_open(name, mode, plist)
        elif mode == 'w':
            self.id = h5f_create(name, plist)
        else:
            raise RuntimeError('Unsupported file open/create mode')

        self.attrs = Attributes(self.id)

        if comm is not None:
            h5p_close(plist)

        self.name = name
        self.opened = True

    def close(self):
        h5f_close(self.id)
        self.opened = False

class Dataset(object):
    """This class defines a HDF5 dataset.

       Attributes:

    ============= =============================================
    Name           Description
    ============= =============================================
    ``shape``     Tuple describing the dataspace of dataset
    ``dtype``     NumPy equivalent datatype of dataset
    ``dataspace`` HDF5 dataspace identifier (integer)
    ``datatype``  HDF5 datatype identifier (integer)
    ``id``        HDF5 identifier of dataset (integer)
    ============= =============================================
"""

    def __init__(self, loc_id, name, dtype=None, shape=None, create=False):
        if create:
            self.shape = shape
            self.dtype = dtype
            self.dataspace = h5s_create(np.asarray(shape))
            self.datatype = h5_type_from_numpy(np.ndarray((1,), dtype))
            self.id = h5d_create(loc_id, name, self.datatype, self.dataspace)
        else:
            self.id = h5d_open(loc_id, name)
            self.dataspace = h5d_get_space(self.id)
            self.datatype = h5d_get_type(self.id)
            self.shape = h5s_get_shape(self.dataspace)
            self.dtype = numpy_type_from_h5(self.datatype)

        self.attrs = Attributes(self.id)

        self.name = name
        self.opened = True

    def write(self, data, selection='all', collective=False):
        """Write NumPy array data into the dataset.

           selection can be a hyperslab instance for writing to a part
           of the dataset or None for no reading
           collective specifies the use of collective IO."""

        if collective:
            plist = h5p_create(H5P_DATASET_XFER)
            h5p_set_dxpl_mpio(plist)
        else:
            plist = H5P_DEFAULT

        filespace = self.dataspace
        memspace = h5s_create(np.asarray(data.shape))
        memtype = h5_type_from_numpy(np.ndarray((1,), data.dtype))

        if selection is None:
            h5s_select_none(memspace)
            h5s_select_none(filespace)

        if isinstance(selection, HyperslabSelection):
            selection.select(self)

        # data array has to be contiguous, if not make a copy
        if not data.flags.contiguous:
            data = data.copy()
        h5d_write(self.id, memtype, memspace, filespace, data, plist)
        
        h5s_close(memspace)
        h5t_close(memtype)
        if collective:
            h5p_close(plist)

    def read(self, data, selection='all', collective=False):
        """Read the dataset into NumPy array data

           selection can be a hyperslab instance for reading part
           of the dataset or None for no reading
           collective specifies the use of collective IO."""

        if collective:
            plist = h5p_create(H5P_DATASET_XFER)
            h5p_set_dxpl_mpio(plist)
        else:
            plist = H5P_DEFAULT

        filespace = self.dataspace
        memspace = h5s_create(np.asarray(data.shape))
        memtype = h5_type_from_numpy(np.ndarray((1,), data.dtype))

        if selection is None:
            h5s_select_none(memspace)
            h5s_select_none(filespace)

        if isinstance(selection, HyperslabSelection):
            selection.select(self)

        # data array has to be contiguous, if not, create a
        # temporary array
        if not data.flags.contiguous:
            data_buf = data.copy()
        else:
            data_buf = data
        h5d_read(self.id, memtype, memspace, filespace, data_buf, plist)

        if not data.flags.contiguous:
            data[:] = data_buf[:]
        
        h5s_close(memspace)
        h5t_close(memtype)
        if collective:
            h5p_close(plist)

    # Interface for access with slicing syntax
    def __setitem__(self, indices, value):
        sel = HyperslabSelection(indices, self.shape)

        if not isinstance(value, np.ndarray):
            value = np.asarray(value)
        if sel.mshape != value.shape:
            raise RuntimeError('Shapes not compatible')
        self.write(value, selection=sel)

    def __getitem__(self, indices):
        sel = HyperslabSelection(indices, self.shape)
        array = np.zeros(sel.mshape, self.dtype)
        self.read(array, sel)
        return array

    def close(self):
        h5t_close(self.datatype)
        h5s_close(self.dataspace)
        h5d_close(self.id)
        self.opened = False
        
    def __del__(self):
        if self.opened:
            self.close()

class Attributes:
    """A dictionary like interface to HDF5 attributes.

       Attributes can be written with the::

          attrs['name'] = value

       and read with the::

          value = attrs['name']

       syntax.
       Values are returned always as NumPy arrays.

    """

    def __init__(self, parent_id):
        self.loc_id = parent_id

    def __setitem__(self, key, data):
        # Should we delete existing attributes?
        # Special treatment for possible None data
        if data is None:
            data = repr(data)

        data = np.asarray(data)
        dataspace = h5s_create(np.asarray(data.shape))
        datatype = h5_type_from_numpy(np.ndarray((1, ), data.dtype))
        id = h5a_create(self.loc_id, key, datatype, dataspace)
        # ensure that data is contiguous
        data = data.copy()
        h5a_write(id, datatype, data)
        h5s_close(dataspace)
        h5t_close(datatype)
        h5a_close(id)

    def __getitem__(self, key):
        id = h5a_open(self.loc_id, key)
        dataspace = h5a_get_space(id)
        datatype = h5a_get_type(id)
        shape = h5s_get_shape(dataspace)
        dtype = numpy_type_from_h5(datatype)
        memtype = h5_type_from_numpy(np.ndarray((1,), dtype))
        data = np.empty(shape, dtype)
        h5a_read(id, memtype, data)
        h5s_close(dataspace)
        h5t_close(datatype)
        h5t_close(memtype)
        h5a_close(id)

        if len(shape) == 0:
            data = np.asscalar(data)
        return data

    def __contains__(self, name):
        return h5a_exists_by_name(self.loc_id, name)

# The following four functions
# _expand_ellipsis, _translate_int, _translate_slice and _handle_simple
# are direct copy-paste from h5py
# Copyright (C) 2008 Andrew Collette
# http://h5py.alfven.org
# License: New BSD
def _expand_ellipsis(args, rank):
    """ Expand ellipsis objects and fill in missing axes.
    """
    n_el = sum(1 for arg in args if arg is Ellipsis)
    if n_el > 1:
        raise ValueError("Only one ellipsis may be used.")
    elif n_el == 0 and len(args) != rank:
        args = args + (Ellipsis,)

    final_args = []
    n_args = len(args)
    for idx, arg in enumerate(args):
        if arg is Ellipsis:
            final_args.extend( (slice(None,None,None),)*(rank-n_args+1) )
        else:
            final_args.append(arg)

    if len(final_args) > rank:
        raise TypeError("Argument sequence too long")

    return final_args

def _translate_int(exp, length):
    """ Given an integer index, return a 3-tuple
        (start, count, step)
        for hyperslab selection
    """
    if exp < 0:
        exp = length+exp

    if not 0<=exp<length:
        raise ValueError("Index (%s) out of range (0-%s)" % (exp, length-1))
    return exp, 1, 1

def _translate_slice(exp, length):
    """ Given a slice object, return a 3-tuple
        (start, count, step)
        for use with the hyperslab selection routines
    """
    start, stop, step = exp.start, exp.stop, exp.step
    if start is None:
        start = 0
    else:
        start = int(start)
    if stop is None:
        stop = length
    else:
        stop = int(stop)
    if step is None:
        step = 1
    else:
        step = int(step)
    if step < 1:
        raise ValueError("Step must be >= 1 (got %d)" % step)
    if stop == start:
        raise ValueError("Zero-length selections are not allowed")
    if stop < start:
        raise ValueError("Reverse-order selections are not allowed")
    if start < 0:
        start = length+start
    if stop < 0:
        stop = length+stop

    if not 0 <= start <= (length-1):
        raise ValueError("Start index %s out of range (0-%d)" % (start,
length-1))
    if not 1 <= stop <= length:
        raise ValueError("Stop index %s out of range (1-%d)" % (stop, length))
    count = (stop-start)//step
    if (stop-start) % step != 0:
        count += 1
    
    if start+count > length:
        raise ValueError("Selection out of bounds (%d; axis has %d)" %
(start+count,length))
    
    return start, count, step
    
def _handle_simple(shape, args):
    """ Process a "simple" selection tuple, containing only slices and
        integer objects.  Return is a 4-tuple with tuples for start,
        count, step, and a flag which tells if the axis is a "scalar"
        selection (indexed by an integer).
    
        If "args" is shorter than "shape", the remaining axes are fully
        selected.
    """
    args = _expand_ellipsis(args, len(shape))

    start = []
    count = []
    step  = []
    scalar = []
    
    for arg, length in zip(args, shape):
        if isinstance(arg, slice):
            x,y,z = _translate_slice(arg, length)
            s = False
        else:
            try:
                x,y,z = _translate_int(int(arg), length)
                s = True
            except TypeError:
                raise TypeError('Illegal index "%s" (must be a slice or number)' % arg)
        start.append(x)
        count.append(y)
        step.append(z)
        scalar.append(s)

    return tuple(start), tuple(count), tuple(step), tuple(scalar)

class HyperslabSelection:
    """This class defines a simple hyperslab selection for a HDF5 dataspace.

    In HDF5, hyperslab is defined by offset, stride, count, and block
    arguments, in the Python side simple indeces or slices are used
    (start, stop, step). The block argument of HDF5 is not used here. """
   
    def __init__(self, indices, shape):
        """Create a hyperslab descriptor."""
        if not isinstance(indices, tuple):
            indices = (indices,)

        dest_shape = shape
        self.indices = indices

        start, count, step, scalar = _handle_simple(dest_shape, indices)
        # This class follows HDF5 convention offset, stride, count
        self.hyperslab = (np.array(start), np.array(step), np.array(count))
        self.mshape = tuple(x for x, y in zip(count, scalar) if not y)

    def select(self, dataset):
        """Make the actual hyperslab selection to the dataspace of dataset."""
        dataspace = dataset.dataspace
        offset, stride, count = self.hyperslab
        h5s_select_hyperslab(dataspace, offset, stride, count, None)




from __future__ import print_function

import pickle
from math import pi
from time import time, ctime

import numpy as np
from ase.io import write
from ase.units import Hartree
from ase.utils import opencew

from gpaw.response.df0 import DF
from gpaw.io.tar import Writer, Reader
from gpaw.response.base import BASECHI
from gpaw.utilities.memory import maxrss
from gpaw.blacs import BlacsGrid, Redistributor
from gpaw.mpi import world, size, rank, serial_comm
from gpaw.response.kernel import calculate_Kc, calculate_Kc_q
from gpaw.response.parallel import parallel_partition, gatherv


class BSE(BASECHI):
    """This class defines Bethe-Salpeter equations."""

    def __init__(self,
                 calc=None,
                 nbands=None,
                 nc=None,
                 nv=None,
                 w=None,
                 q=None,
                 eshift=None,
                 ecut=10.,
                 eta=0.2,
                 gw_skn=None, # GW QP energies in Hartree
                 rpad=None,
                 vcut=None,   # Coulomb cutoff only 2D works
                 ftol=1e-5,
                 txt=None,
                 optical_limit=None,
                 integrate_coulomb=None,
                 print_coulomb=False,
                 coupling=False,  # False : use Tamm-Dancoff Approx
                 mode='BSE',      # BSE, TDHF or RPA
                 kernel_file=None,#'W_qGG',
                 qsymm=True): 

        BASECHI.__init__(self, calc=calc, nbands=nbands, w=w, q=q,
                         eshift=eshift, ecut=ecut, eta=eta, rpad=rpad,
                         ftol=ftol, txt=txt, optical_limit=optical_limit)

        assert mode in ['RPA', 'TDHF', 'BSE']

        self.epsilon_w = None
        self.coupling = coupling
        self.vcut = vcut
        self.nc = nc # conduction band index
        self.nv = nv # valence band index
        self.gw_skn = gw_skn
        self.mode = mode
        self.integrate_coulomb = integrate_coulomb
        self.print_coulomb = print_coulomb
        self.kernel_file = kernel_file
        self.qsymm = qsymm

    def initialize(self):

        self.printtxt('----------------------------------------')
        self.printtxt('Bethe-Salpeter Equation calculation')
        self.printtxt('----------------------------------------')
        self.printtxt('Started at:  %s' % ctime())
        self.printtxt('')
        BASECHI.initialize(self)
        assert self.nspins == 1
        
        calc = self.calc
        self.kd = kd = calc.wfs.kd

        # frequency points init
        self.dw = self.w_w[1] - self.w_w[0]
        assert ((self.w_w[1:] - self.w_w[:-1] - self.dw) < 1e-10).all() # make sure its linear w grid
        assert self.w_w.max() == self.w_w[-1]

        self.dw /= Hartree
        self.w_w  /= Hartree
        self.wmax = self.w_w[-1] 
        self.Nw  = int(self.wmax / self.dw) + 1

        # band init
        if self.nc is None:
            nv = self.nvalence / 2 - 1
            self.nv = np.array([nv, nv+1]) # conduction band start / end
            self.nc = np.array([nv+1, nv+2]) # valence band start / end

        self.printtxt('')
        self.printtxt('Number of electrons      : %d' % (self.nvalence))
        self.printtxt('Valence band included    : (band %d to band %d)' %(self.nv[0], self.nv[1]-1))
        self.printtxt('Conduction band included : (band %d to band %d)' %(self.nc[0], self.nc[1]-1))
        if self.eshift is not None:
            self.printtxt('Scissors operator        : %2.3f eV' % self.eshift)
        self.printtxt('')
            
        # find the pair index and initialized pair energy (e_i - e_j) and occupation(f_i-f_j)
        self.e_S = {}
        focc_s = {}
        self.Sindex_S3 = {}
        iS = 0
        kq_k = self.kq_k
        for k1 in range(self.kd.nbzkpts):
            ibzkpt1 = kd.bz2ibz_k[k1]
            ibzkpt2 = kd.bz2ibz_k[kq_k[k1]]
            for n1 in range(self.nv[0], self.nv[1]): 
                for m1 in range(self.nc[0], self.nc[1]): 
                    focc = self.f_skn[0][ibzkpt1,n1] - self.f_skn[0][ibzkpt2,m1]
                    if self.coupling: # Don't use Tamm-Dancoff Approx.
                        check_ftol = np.abs(focc) > self.ftol
                    else:
                        check_ftol = focc > self.ftol
                    if check_ftol:
                        if self.gw_skn is None:
                            self.e_S[iS] = self.e_skn[0][ibzkpt2,m1] - self.e_skn[0][ibzkpt1,n1]
                        else:
                            self.e_S[iS] = self.gw_skn[0][ibzkpt2,m1] - self.gw_skn[0][ibzkpt1,n1]
                            
                        focc_s[iS] = focc
                        self.Sindex_S3[iS] = (k1, n1, m1)
                        iS += 1
        self.nS = iS
        self.focc_S = np.zeros(self.nS)
        for iS in range(self.nS):
            self.focc_S[iS] = focc_s[iS]

        # q points init
        self.bzq_qc = kd.get_bz_q_points()
        if not self.qsymm:
            self.ibzq_qc = self.bzq_qc
        else:
            (self.ibzq_qc, self.ibzq_q, self.iop_q,
             self.timerev_q, self.diff_qc) = kd.get_ibz_q_points(self.bzq_qc,
                                                                 calc.wfs.kd.symmetry.op_scc)
            if np.abs(self.bzq_qc - kd.bzk_kc).sum() < 1e-8:
                assert np.abs(self.ibzq_qc - kd.ibzk_kc).sum() < 1e-8
        self.nibzq = len(self.ibzq_qc)

        # Parallel initialization 
        # kcomm and wScomm is only to be used when wavefunctions are distributed in parallel.
        self.comm = self.Scomm = world
        self.kcomm = world
        self.wScomm = serial_comm
        self.nS, self.nS_local, self.nS_start, self.nS_end = parallel_partition(
            self.nS, world.rank, world.size, reshape=False)

        self.print_bse()

        if calc.input_parameters['mode'] == 'lcao':
            calc.initialize_positions()

        # Coulomb interaction at q=0 for Hartree coupling
        ### 2D z direction only !!!!!!!!!!!!!!!!!!!!!!
        if self.integrate_coulomb is None:
            self.integrate_coulomb = []
            if self.vcut is None:
                pass
            elif self.vcut == '2D':
                for iG in range(len(self.Gvec_Gc)):
                    if self.Gvec_Gc[iG, 0] == 0 and self.Gvec_Gc[iG, 1] == 0:
                        self.integrate_coulomb.append(iG)
            else:
                raise NotImplementedError
        elif isinstance(self.integrate_coulomb, int):
            self.integrate_coulomb = range(self.integrate_coulomb)
        elif self.integrate_coulomb == 'all':
            self.integrate_coulomb = range(len(self.Gvec_Gc))
        elif isinstance(self.integrate_coulomb, list):
            pass
        else:
            raise 'Invalid option for integrate_coulomb'
        
        self.printtxt('')
        self.printtxt('Calculating bare Coulomb kernel')
        if not len(self.integrate_coulomb) == 0:
            self.printtxt('Integrating Coulomb kernel at %s reciprocal lattice vector(s)' % len(self.integrate_coulomb))
        
        # Coulomb interaction at problematic G's for exchange coupling
        if len(self.integrate_coulomb) != 0:
            self.vint_Gq = []
            for iG in self.integrate_coulomb:
                v_q, v0_q = calculate_Kc_q(self.acell_cv,
                                           self.bcell_cv,
                                           self.pbc,
                                           self.kd.N_c,
                                           vcut=self.vcut,
                                           Gvec_c=self.Gvec_Gc[iG],
                                           q_qc=self.ibzq_qc.copy())
                self.vint_Gq.append(v_q)                    
                if self.print_coulomb:
                    self.printtxt('')
                    self.printtxt('Average kernel relative to bare kernel - \int v(q)dq / v(q0): ')
                    self.printtxt('  G: % s' % self.Gvec_Gc[iG])
                    for iq in range(len(v_q)):
                        q_s = '    q = [%1.2f,  %1.2f,  %1.2f]: ' % (self.ibzq_qc[iq,0],
                                                                     self.ibzq_qc[iq,1],
                                                                     self.ibzq_qc[iq,2])
                        v_rel = v_q[iq] / v0_q[iq]
                        self.printtxt(q_s + '%1.3f' % v_rel)
        self.printtxt('')

        self.V_qGG = self.full_bare_interaction()

    def calculate(self):
        calc = self.calc
        focc_S = self.focc_S
        e_S = self.e_S
        op_scc = calc.wfs.kd.symmetry.op_scc

        # Get phi_qaGp
        if self.mode == 'RPA':
            self.phi_aGp = self.get_phi_aGp()
        else:
            fd = opencew('phi_qaGp')
            if fd is None:
                self.reader = Reader('phi_qaGp')
                tmp = self.load_phi_aGp(self.reader, 0)[0]
                assert len(tmp) == self.npw
                self.printtxt('Finished reading phi_aGp')
            else:
                self.printtxt('Calculating phi_qaGp')
                self.get_phi_qaGp()
                world.barrier()
                self.reader = Reader('phi_qaGp')                
            self.printtxt('Memory used %f M' % (maxrss() / 1024.**2))
            self.printtxt('')

        if self.optical_limit:
            iq = np.where(np.sum(abs(self.ibzq_qc), axis=1) < 1e-5)[0][0]
        else:
            iq = np.where(np.sum(abs(self.ibzq_qc - self.q_c), axis=1) < 1e-5)[0][0]
        kc_G = np.array([self.V_qGG[iq, iG, iG] for iG in range(self.npw)])
        if self.optical_limit:
            kc_G[0] = 0.

        # Get screened Coulomb kernel
        if self.mode == 'BSE':
            try:
                # Read
                data = pickle.load(open(self.kernel_file+'.pckl'))
                W_qGG = data['W_qGG']
                assert np.shape(W_qGG) == np.shape(self.V_qGG)
                self.printtxt('Finished reading screening interaction kernel')
            except:
                # Calculate from scratch
                self.printtxt('Calculating screening interaction kernel.')
                W_qGG = self.full_static_screened_interaction()
            self.printtxt('')
        else:
            W_qGG = self.V_qGG
 
        t0 = time()
        self.printtxt('Calculating %s matrix elements' % self.mode)

        # Calculate full kernel
        K_SS = np.zeros((self.nS_local, self.nS), dtype=complex)
        self.rhoG0_S = np.zeros(self.nS, dtype=complex)

        #noGmap = 0
        for iS in range(self.nS_start, self.nS_end):
            k1, n1, m1 = self.Sindex_S3[iS]
            rho1_G = self.density_matrix(n1,m1,k1)
            self.rhoG0_S[iS] = rho1_G[0]
            for jS in range(self.nS):
                k2, n2, m2 = self.Sindex_S3[jS]
                rho2_G = self.density_matrix(n2,m2,k2)
                K_SS[iS-self.nS_start, jS] = np.sum(rho1_G.conj() * rho2_G * kc_G)

                if not self.mode == 'RPA':
                    rho3_G = self.density_matrix(n1,n2,k1,k2)
                    rho4_G = self.density_matrix(m1,m2,self.kq_k[k1],
                                                 self.kq_k[k2])

                    q_c = self.kd.bzk_kc[k2] - self.kd.bzk_kc[k1]
                    q_c[np.where(q_c > 0.501)] -= 1.
                    q_c[np.where(q_c < -0.499)] += 1.
                    iq = self.kd.where_is_q(q_c, self.bzq_qc)
                    
                    if not self.qsymm:    
                        W_GG = W_qGG[iq]
                    else:
                        ibzq = self.ibzq_q[iq]
                        W_GG_tmp = W_qGG[ibzq]

                        iop = self.iop_q[iq]
                        timerev = self.timerev_q[iq]
                        diff_c = self.diff_qc[iq]
                        invop = np.linalg.inv(op_scc[iop])
                        Gindex = np.zeros(self.npw, dtype=int)
                        for iG in range(self.npw):
                            G_c = self.Gvec_Gc[iG]
                            if timerev:
                                RotG_c = -np.int8(np.dot(invop, G_c+diff_c).round())
                            else:
                                RotG_c = np.int8(np.dot(invop, G_c+diff_c).round())
                            tmp_G = np.abs(self.Gvec_Gc - RotG_c).sum(axis=1)
                            try:
                                Gindex[iG] = np.where(tmp_G < 1e-5)[0][0]
                            except:
                                #noGmap += 1
                                Gindex[iG] = -1
    
                        W_GG = np.zeros_like(W_GG_tmp)
                        for iG in range(self.npw):
                            for jG in range(self.npw):
                                if Gindex[iG] == -1 or Gindex[jG] == -1:
                                    W_GG[iG, jG] = 0
                                else:
                                    W_GG[iG, jG] = W_GG_tmp[Gindex[iG], Gindex[jG]]
                                    
                    if self.mode == 'BSE':
                        tmp_GG = np.outer(rho3_G.conj(), rho4_G) * W_GG
                        K_SS[iS-self.nS_start, jS] -= 0.5 * np.sum(tmp_GG)
                    else:
                        tmp_G = rho3_G.conj() * rho4_G * np.diag(W_GG)
                        K_SS[iS-self.nS_start, jS] -= 0.5 * np.sum(tmp_G)
            self.timing(iS, t0, self.nS_local, 'pair orbital') 
 
        K_SS /= self.vol

        world.sum(self.rhoG0_S)
        #self.printtxt('Number of G indices outside the Gvec_Gc: %d' % noGmap)

        # Get and solve Hamiltonian
        H_sS = np.zeros_like(K_SS)
        for iS in range(self.nS_start, self.nS_end):
            H_sS[iS-self.nS_start,iS] = e_S[iS]
            for jS in range(self.nS):
                H_sS[iS-self.nS_start,jS] += focc_S[iS] * K_SS[iS-self.nS_start,jS]
  
        # Force matrix to be Hermitian
        if not self.coupling:
            if world.size > 1:
                H_Ss = self.redistribute_H(H_sS)
            else:
                H_Ss = H_sS
            H_sS = (np.real(H_sS) + np.real(H_Ss.T)) / 2. + 1j * (np.imag(H_sS) - np.imag(H_Ss.T)) /2.

        # Save H_sS matrix
        self.par_save('H_SS','H_SS', H_sS)

        return H_sS

    def diagonalize(self, H_sS):
        if self.coupling: # Non-Hermitian matrix can only use linalg.eig
            self.printtxt('Use numpy.linalg.eig')
            H_SS = np.zeros((self.nS, self.nS), dtype=complex)
            if self.nS % world.size == 0:
                world.all_gather(H_sS, H_SS)
            else:
                H_SS = gatherv(H_sS)

            self.w_S, self.v_SS = np.linalg.eig(H_SS)
            self.par_save('v_SS', 'v_SS', self.v_SS[self.nS_start:self.nS_end, :].copy())
        else:
            if world.size == 1:
                self.printtxt('Use lapack.')
                from gpaw.utilities.lapack import diagonalize
                self.w_S = np.zeros(self.nS)
                H_SS = H_sS
                diagonalize(H_SS, self.w_S)
                self.v_SS = H_SS.conj() # eigenvectors in the rows, transposed later
            else:
                self.printtxt('Use scalapack')
                self.w_S, self.v_sS = self.scalapack_diagonalize(H_sS)
                self.v_SS = self.v_sS # just use the same name
            self.par_save('v_SS', 'v_SS', self.v_SS)
        
        return 


    def par_save(self,filename, name, A_sS):
        from gpaw.io import open 

        nS = self.nS
        
        if rank == 0:
            w = open(filename, 'w', world)
            w.dimension('nS', nS)
            
            if name == 'v_SS':
                w.add('w_S', ('nS',), dtype=self.w_S.dtype)
                w.fill(self.w_S)
            w.add('rhoG0_S', ('nS',), dtype=complex)
            w.fill(self.rhoG0_S)
            w.add(name, ('nS', 'nS'), dtype=complex)

            tmp = np.zeros_like(A_sS)

        # Assumes that H_SS is written in order from rank 0 - rank N
        for irank in range(size):
            if irank == 0:
                if rank == 0:
                    w.fill(A_sS)
            else:
                if rank == irank:
                    world.send(A_sS, 0, irank+100)
                if rank == 0:
                    world.receive(tmp, irank, irank+100)
                    w.fill(tmp)
        if rank == 0:
            w.close()
        world.barrier()

    def par_load(self,filename, name):
        from gpaw.io import open 
        r = open(filename, 'r')
        nS = r.dimension('nS')

        if name == 'v_SS':
            self.w_S = r.get('w_S')

        A_SS = np.zeros((self.nS_local, nS), dtype=complex)
        for iS in range(self.nS_start, self.nS_end):   
            A_SS[iS-self.nS_start,:] = r.get(name, iS)

        self.rhoG0_S = r.get('rhoG0_S')

        r.close()
        
        return A_SS
    

    def full_static_screened_interaction(self):
        """Calcuate W_GG(q)"""
        W_qGG = np.zeros((self.nibzq, self.npw, self.npw), dtype=complex)

        t0 = time()
        for iq in range(self.nibzq):
            q = self.ibzq_qc[iq]
            optical_limit = False
            if np.abs(q).sum() < 1e-8:
                q = np.array([0.0001, 0, 0])
                optical_limit = True
            df = DF(calc=self.calc,
                    q=q,
                    w=(0.,),
                    optical_limit=optical_limit,
                    nbands=self.nbands,
                    hilbert_trans=False,
                    eta=0.0001,
                    ecut=self.ecut*Hartree,
                    xc='RPA',
                    txt='df.out')
            df.initialize()
            df.calculate()

            if optical_limit:
                K_GG = self.V_qGG[iq].copy()
                K0 = calculate_Kc(q,
                                  self.Gvec_Gc,
                                  self.acell_cv,
                                  self.bcell_cv,
                                  self.pbc,
                                  vcut=self.vcut)[0,0]

                for iG in range(1,self.npw):
                    K_GG[0, iG] = self.V_qGG[iq, iG, iG]**0.5 * K0**0.5
                    K_GG[iG, 0] = self.V_qGG[iq, iG, iG]**0.5 * K0**0.5
                K_GG[0,0] = K0
                df_GG = np.eye(self.npw, self.npw) - K_GG*df.chi0_wGG[0]
            else:
                df_GG = np.eye(self.npw, self.npw) - self.V_qGG[iq]*df.chi0_wGG[0]
            dfinv_GG = np.linalg.inv(df_GG)
            
            if optical_limit:
                eps = 1/dfinv_GG[0,0]
                self.printtxt('    RPA macroscopic dielectric constant is: %3.3f' %  eps.real)
            W_qGG[iq] = dfinv_GG * self.V_qGG[iq]
            self.timing(iq, t0, self.nibzq, 'iq')
            
        if rank == 0:
            if self.kernel_file is not None:
                data = {'W_qGG': W_qGG}
                name = self.kernel_file+'.pckl'
                pickle.dump(data, open(name, 'w'), -1)
        
        return W_qGG


    def full_bare_interaction(self):
        """Calculate V_GG(q)"""

        V_qGG = np.zeros((self.nibzq, self.npw, self.npw), dtype=complex)
        
        for iq in range(self.nibzq):
            q = self.ibzq_qc[iq]

            Vq_G = np.diag(calculate_Kc(q,
                                        self.Gvec_Gc,
                                        self.acell_cv,
                                        self.bcell_cv,
                                        self.pbc,
                                        integrate_gamma=True,
                                        N_k=self.kd.N_c,
                                        vcut=self.vcut))**0.5
            
            for i, iG in enumerate(self.integrate_coulomb):
                Vq_G[iG] = self.vint_Gq[i][iq]**0.5

            V_qGG[iq] = np.outer(Vq_G, Vq_G)

        return V_qGG

                                          
    def print_bse(self):

        printtxt = self.printtxt

        if not self.mode == 'RPA':
            printtxt('Number of q points           : %d' %(self.nibzq))
        printtxt('Number of frequency points   : %d' %(self.Nw) )
        printtxt('Number of pair orbitals      : %d' %(self.nS) )
        printtxt('')
        printtxt('Parallelization scheme:')
        printtxt('   Total cpus         : %d' %(world.size))
        printtxt('   pair orb parsize   : %d' %(self.Scomm.size))        
        
        return


    def get_phi_qaGp(self):

        N1_max = 0
        N2_max = 0
        natoms = len(self.calc.wfs.setups)
        for id in range(natoms):
            N1 = self.npw
            N2 = self.calc.wfs.setups[id].ni**2
            if N1 > N1_max:
                N1_max = N1
            if N2 > N2_max:
                N2_max = N2
        
        nbzq = self.kd.nbzkpts
        nbzq, nq_local, q_start, q_end = parallel_partition(
                                   nbzq, world.rank, world.size, reshape=False)
        phimax_qaGp = np.zeros((nq_local, natoms, N1_max, N2_max), dtype=complex)
        #phimax_qaGp = np.zeros((nbzq, natoms, N1_max, N2_max), dtype=complex)

        t0 = time()
        for iq in range(nq_local):
            q_c = self.bzq_qc[iq + q_start]
            tmp_aGp = self.get_phi_aGp(q_c, parallel=False)
            for id in range(natoms):
                N1, N2 = tmp_aGp[id].shape
                phimax_qaGp[iq, id, :N1, :N2] = tmp_aGp[id]
            self.timing(iq*world.size, t0, nq_local, 'iq')
        world.barrier()

        # Write to disk
        filename = 'phi_qaGp'
        if world.rank == 0:
            w = Writer(filename)
            w.dimension('nbzq', nbzq)
            w.dimension('natoms', natoms)
            w.dimension('nG', N1_max)
            w.dimension('nii', N2_max)
            w.add('phi_qaGp', ('nbzq', 'natoms', 'nG', 'nii',), dtype=complex)

        for q in range(nbzq):
            residual = nbzq % size
            N_local = nbzq // size
            if q < residual * (N_local + 1):
                qrank = q // (N_local + 1)
            else:
                qrank = (q - residual * (N_local + 1)) // N_local + residual
                
            if qrank == 0:
                if world.rank == 0:
                    phi_aGp = phimax_qaGp[q - q_start]
            else:
                if world.rank == qrank:
                    phi_aGp = phimax_qaGp[q - q_start]
                    world.send(phi_aGp, 0, q)
                elif world.rank == 0:
                    world.receive(phi_aGp, qrank, q)
            if world.rank == 0:
                w.fill(phi_aGp)
        if world.rank == 0:
            w.close()
        world.barrier()

    def load_phi_aGp(self, reader, iq):

        phimax_aGp = np.array(reader.get('phi_qaGp', iq), complex)

        phi_aGp = {}
        natoms = len(phimax_aGp)
        for a in range(natoms):
            N1 = self.npw
            N2 = self.calc.wfs.setups[a].ni**2
            phi_aGp[a] = phimax_aGp[a, :N1, :N2]

        return phi_aGp

    def get_dielectric_function(self, filename='df.dat', readfile=None):

        if self.epsilon_w is None:
            self.initialize()

            if readfile is None:
                H_sS = self.calculate()
                self.printtxt('Diagonalizing %s matrix.' % self.mode)
                self.diagonalize(H_sS)
                self.printtxt('Calculating dielectric function.')
            elif readfile == 'H_SS':
                H_sS = self.par_load('H_SS', 'H_SS')
                self.printtxt('Finished reading H_SS.gpw')
                self.diagonalize(H_sS)
                self.printtxt('Finished diagonalizing BSE matrix')
            elif readfile == 'v_SS':
                self.v_SS = self.par_load('v_SS', 'v_SS')
                self.printtxt('Finished reading v_SS.gpw')
            else:
                1 / 0

            w_S = self.w_S
            if not self.coupling:
                v_SS = self.v_SS.T # v_SS[:,lamda]
            else:
                v_SS = self.v_SS
            rhoG0_S = self.rhoG0_S
            focc_S = self.focc_S

            # get overlap matrix
            if self.coupling:
                tmp = np.dot(v_SS.conj().T, v_SS )
                overlap_SS = np.linalg.inv(tmp)
    
            # get chi
            epsilon_w = np.zeros(self.Nw, dtype=complex)

            A_S = np.dot(rhoG0_S, v_SS)
            B_S = np.dot(rhoG0_S*focc_S, v_SS)
            if self.coupling:
                C_S = np.dot(B_S.conj(), overlap_SS.T) * A_S
            else:
                if world.size == 1:
                    C_S = B_S.conj() * A_S
                else:
                    tmp = B_S.conj() * A_S
                    C_S = gatherv(tmp, self.nS)

            for iw in range(self.Nw):
                tmp_S = 1. / (iw*self.dw - w_S + 1j*self.eta)
                epsilon_w[iw] += np.dot(tmp_S, C_S)
    
            epsilon_w *=  - 4 * pi / np.inner(self.qq_v, self.qq_v) / self.vol
            epsilon_w += 1        

            self.epsilon_w = epsilon_w
    
        if rank == 0:
            f = open(filename, 'w')
            for iw in range(self.Nw):
                energy = iw * self.dw * Hartree
                print(energy, np.real(epsilon_w[iw]), np.imag(epsilon_w[iw]),
                      file=f)
            f.close()
            #g.close()
        # Wait for I/O to finish
        world.barrier()

        """Check f-sum rule."""
        N1 = 0
        for iw in range(self.Nw):
            w = iw * self.dw
            N1 += np.imag(epsilon_w[iw]) * w
        N1 *= self.dw * self.vol / (2 * pi**2)

        self.printtxt('')
        self.printtxt('Sum rule:')
        nv = self.nvalence
        self.printtxt('N1 = %f, %f  %% error' %(N1, (N1 - nv) / nv * 100) )

        return epsilon_w


    def get_e_h_density(self, lamda=None, filename=None):

        if filename is not None:
            self.load(filename)
            self.initialize()
            
        gd = self.gd
        v_SS = self.v_SS
        A_S = v_SS[:, lamda]
        kq_k = self.kq_k
        kd = self.kd

        # Electron density
        nte_R = gd.zeros()
        
        for iS in range(self.nS_start, self.nS_end):
            print('electron density:', iS)
            k1, n1, m1 = self.Sindex_S3[iS]
            ibzkpt1 = kd.bz2ibz_k[k1]
            psitold_g = self.get_wavefunction(ibzkpt1, n1)
            psit1_g = kd.transform_wave_function(psitold_g, k1)

            for jS in range(self.nS):
                k2, n2, m2 = self.Sindex_S3[jS]
                if m1 == m2 and k1 == k2:
                    psitold_g = self.get_wavefunction(ibzkpt1, n2)
                    psit2_g = kd.transform_wave_function(psitold_g, k1)

                    nte_R += A_S[iS] * A_S[jS].conj() * psit1_g.conj() * psit2_g

        # Hole density
        nth_R = gd.zeros()
        
        for iS in range(self.nS_start, self.nS_end):
            print('hole density:', iS)
            k1, n1, m1 = self.Sindex_S3[iS]
            ibzkpt1 = kd.bz2ibz_k[kq_k[k1]]
            psitold_g = self.get_wavefunction(ibzkpt1, m1)
            psit1_g = kd.transform_wave_function(psitold_g, kq_k[k1])

            for jS in range(self.nS):
                k2, n2, m2 = self.Sindex_S3[jS]
                if n1 == n2 and k1 == k2:
                    psitold_g = self.get_wavefunction(ibzkpt1, m2)
                    psit2_g = kd.transform_wave_function(psitold_g, kq_k[k1])

                    nth_R += A_S[iS] * A_S[jS].conj() * psit1_g * psit2_g.conj()
                    
        self.Scomm.sum(nte_R)
        self.Scomm.sum(nth_R)


        if rank == 0:
            write('rho_e.cube',self.calc.atoms, format='cube', data=nte_R)
            write('rho_h.cube',self.calc.atoms, format='cube', data=nth_R)
            
        world.barrier()
        
        return 

    def get_excitation_wavefunction(self, lamda=None,filename=None, re_c=None, rh_c=None):
        """ garbage at the moment. come back later"""
        if filename is not None:
            self.load(filename)
            self.initialize()
            
        gd = self.gd
        v_SS = self.v_SS
        A_S = v_SS[:, lamda]
        kq_k = self.kq_k
        kd = self.kd

        nx, ny, nz = self.gd.N_c
        nR = 9
        nR2 = (nR - 1 ) // 2
        if re_c is not None:
            psith_R = gd.zeros(dtype=complex)
            psith2_R = np.zeros((nR*nx, nR*ny, nz), dtype=complex)
            
        elif rh_c is not None:
            psite_R = gd.zeros(dtype=complex)
            psite2_R = np.zeros((nR*nx, ny, nR*nz), dtype=complex)
        else:
            self.printtxt('No wavefunction output !')
            return
            
        for iS in range(self.nS_start, self.nS_end):

            k, n, m = self.Sindex_S3[iS]
            ibzkpt1 = kd.bz2ibz_k[k]
            ibzkpt2 = kd.bz2ibz_k[kq_k[k]]
            print('hole wavefunction', iS, (k,n,m),A_S[iS])
            
            psitold_g = self.get_wavefunction(ibzkpt1, n)
            psit1_g = kd.transform_wave_function(psitold_g, k)

            psitold_g = self.get_wavefunction(ibzkpt2, m)
            psit2_g = kd.transform_wave_function(psitold_g, kq_k[k])

            if re_c is not None:
                # given electron position, plot hole wavefunction
                tmp = A_S[iS] * psit1_g[re_c].conj() * psit2_g
                psith_R += tmp

                k_c = self.kd.bzk_kc[k] + self.q_c
                for i in range(nR):
                    for j in range(nR):
                        R_c = np.array([i-nR2, j-nR2, 0])
                        psith2_R[i*nx:(i+1)*nx, j*ny:(j+1)*ny, 0:nz] += \
                                                tmp * np.exp(1j*2*pi*np.dot(k_c,R_c))
                
            elif rh_c is not None:
                # given hole position, plot electron wavefunction
                tmp = A_S[iS] * psit1_g.conj() * psit2_g[rh_c] * self.expqr_g
                psite_R += tmp

                k_c = self.kd.bzk_kc[k]
                k_v = np.dot(k_c, self.bcell_cv)
                for i in range(nR):
                    for j in range(nR):
                        R_c = np.array([i-nR2, 0, j-nR2])
                        R_v = np.dot(R_c, self.acell_cv)
                        assert np.abs(np.dot(k_v, R_v) - np.dot(k_c, R_c) * 2*pi).sum() < 1e-5
                        psite2_R[i*nx:(i+1)*nx, 0:ny, j*nz:(j+1)*nz] += \
                                                tmp * np.exp(-1j*np.dot(k_v,R_v))
                
            else:
                pass

        if re_c is not None:
            self.Scomm.sum(psith_R)
            self.Scomm.sum(psith2_R)
            if rank == 0:
                write('psit_h.cube',self.calc.atoms, format='cube', data=psith_R)

                atoms = self.calc.atoms
                shift = atoms.cell[0:2].copy()
                atoms.cell[0:2] *= nR2
                atoms.positions += shift * (nR2 - 1)
                
                write('psit_bigcell_h.cube',atoms, format='cube', data=psith2_R)
        elif rh_c is not None:
            self.Scomm.sum(psite_R)
            self.Scomm.sum(psite2_R)
            if rank == 0:
                write('psit_e.cube',self.calc.atoms, format='cube', data=psite_R)

                atoms = self.calc.atoms
#                shift = atoms.cell[0:2].copy()
                atoms.cell[0:2] *= nR2
#                atoms.positions += shift * (nR2 - 1)
                
                write('psit_bigcell_e.cube',atoms, format='cube', data=psite2_R)
                
        else:
            pass

        world.barrier()
            
        return
    

    def load(self, filename):

        data = pickle.load(open(filename))
        self.w_S  = data['w_S']
        self.v_SS = data['v_SS']

        self.printtxt('Read successfully !')
        

    def save(self, filename):
        """Dump essential data"""

        data = {'w_S'  : self.w_S,
                'v_SS' : self.v_SS}
        
        if rank == 0:
            pickle.dump(data, open(filename, 'w'), -1)

        world.barrier()

    def redistribute_H(self, H_sS):

        g1 = BlacsGrid(world, size, 1)
        g2 = BlacsGrid(world, 1, size)
        N = self.nS
        nndesc1 = g1.new_descriptor(N, N, self.nS_local,  N) 
        nndesc2 = g2.new_descriptor(N, N, N, self.nS_local)
        
        H_Ss = nndesc2.empty(dtype=H_sS.dtype)
        redistributor = Redistributor(world, nndesc1, nndesc2)
        redistributor.redistribute(H_sS, H_Ss)

        return H_Ss

    def scalapack_diagonalize(self, H_sS):

        mb = 32
        N = self.nS
        
        g1 = BlacsGrid(world, size,    1)
        g2 = BlacsGrid(world, size//2, 2)
        nndesc1 = g1.new_descriptor(N, N, self.nS_local,  N) 
        nndesc2 = g2.new_descriptor(N, N, mb, mb)
        
        A_ss = nndesc2.empty(dtype=H_sS.dtype)
        redistributor = Redistributor(world, nndesc1, nndesc2)
        redistributor.redistribute(H_sS, A_ss)
        
        # diagonalize
        v_ss = nndesc2.zeros(dtype=A_ss.dtype)
        w_S = np.zeros(N,dtype=float)
        nndesc2.diagonalize_dc(A_ss, v_ss, w_S, 'L')
        
        # distribute the eigenvectors to master
        v_sS = np.zeros_like(H_sS)
        redistributor = Redistributor(world, nndesc2, nndesc1)
        redistributor.redistribute(v_ss, v_sS)

#        v2_SS = np.zeros((self.nS, self.nS), dtype=complex)
#        world.all_gather(v_sS, v2_SS)
        
        return w_S, v_sS.conj()

    """
    def get_chi(self, w):
        H_SS = self.calculate()
        self.printtxt('Diagonalizing BSE matrix.')
        self.diagonalize(H_SS)
        self.printtxt('Calculating BSE response function.')

        w_S = self.w_S
        if not self.coupling:
            v_SS = self.v_SS.T # v_SS[:,lamda]
        else:
            v_SS = self.v_SS
        rhoG0_S = self.rhoG0_S
        focc_S = self.focc_S

        # get overlap matrix
        if self.coupling:
            tmp = np.dot(v_SS.conj().T, v_SS )
            overlap_SS = np.linalg.inv(tmp)
    
        # get chi
        chi_wGG = np.zeros((len(w), self.npw, self.npw), dtype=complex)
        t0 = time()

        A_S = np.dot(rhoG0_S, v_SS)
        B_S = np.dot(rhoG0_S*focc_S, v_SS)
        if self.coupling:
            C_S = np.dot(B_S.conj(), overlap_SS.T) * A_S
        else:
            C_S = B_S.conj() * A_S

        return chi_wGG
    """



from __future__ import print_function
from ase.dft.kpoints import monkhorst_pack
from ase.lattice import bulk
from gpaw import GPAW, FermiDirac
from gpaw.wavefunctions.pw import PW


nk = 4
for mode in ('pw', 'fd'):
    kpts = monkhorst_pack((nk, nk, nk))
    kshift = 1.0 / (2 * nk)
    kpts += kshift

    atoms = bulk('MgO', 'rocksalt', a=4.212)

    if mode == 'pw':
        calc = GPAW(mode=PW(800), basis='dzp', xc='PBE', maxiter=300,
                    kpts=kpts, parallel={'band': 1, 'domain': 1},
                    setups={'Mg': '2'},
                    occupations=FermiDirac(0.01))
        atoms.set_calculator(calc)
        E1 = atoms.get_potential_energy()
        from gpaw.xc.hybridg import HybridXC
        exx = HybridXC('EXX', method='acdf')
        E_hf1 = E1 + calc.get_xc_difference(exx)
        
    else:
        calc = GPAW(h=0.12,
                    basis='dzp', kpts=kpts, xc='PBE',
                    setups={'Mg': '2'},
                    parallel={'domain': 1, 'band': 1},
                    occupations=FermiDirac(0.01))
        atoms.set_calculator(calc)
        E2 = atoms.get_potential_energy()
        from gpaw.xc.hybridk import HybridXC
        exx = HybridXC('EXX', acdf=True)
        E_hf2 = E2 + calc.get_xc_difference(exx)

print((E1, E2, E_hf1, E_hf2))
assert abs(E1 - E2) < 0.05
assert abs(E_hf1 - E_hf2) < 0.05



import numpy as np

from ase.units import Hartree, Bohr
from ase.dft.kpoints import monkhorst_pack

import gpaw
import gpaw.mpi as mpi
from gpaw.wavefunctions.pw import PW
from gpaw.occupations import FermiDirac
from gpaw.poisson import PoissonSolver, FFTPoissonSolver


def usesymm2symmetry(usesymm):
    if usesymm is None:
        return {'time_reversal': False, 'point_group': False}
    if usesymm:
        return {'time_reversal': True, 'point_group': True}
    return {'time_reversal': True, 'point_group': False}


class InputParameters(dict):
    def __init__(self, **kwargs):
        dict.__init__(self, [
            ('h', None),  # Angstrom
            ('xc', 'LDA'),
            ('gpts', None),
            ('kpts', [(0.0, 0.0, 0.0)]),
            ('lmax', 2),
            ('charge', 0),
            ('fixmom', False),  # don't use this
            ('nbands', None),
            ('setups', 'paw'),
            ('basis', {}),
            ('width', None),  # eV, don't use this
            ('occupations', None),
            ('spinpol', None),
            ('usesymm', 'default'),  # don't use this
            ('stencils', (3, 3)),
            ('fixdensity', False),
            ('mixer', None),
            ('txt', '-'),
            ('hund', False),
            ('random', False),
            ('dtype', None),
            ('filter', None),
            ('maxiter', 333),  # google it's spiritual meaning!
            ('parallel', {'kpt': None,
                          'domain': gpaw.parsize_domain,
                          'band': gpaw.parsize_bands,
                          'order': 'kdb',
                          'stridebands': False,
                          # Distribute density/potential on world.
                          # What is a better name for this parameter?
                          # It should probably accept a parsize tuple
                          'augment_grids': gpaw.augment_grids,
                          'sl_auto': False,
                          'sl_default': gpaw.sl_default,
                          'sl_diagonalize': gpaw.sl_diagonalize,
                          'sl_inverse_cholesky': gpaw.sl_inverse_cholesky,
                          'sl_lcao': gpaw.sl_lcao,
                          'sl_lrtddft': gpaw.sl_lrtddft,
                          'buffer_size': gpaw.buffer_size}),
            ('parsize', None),  # don't use this
            ('parsize_bands', None),  # don't use this
            ('parstride_bands', False),  # don't use this
            ('external', None),  # eV
            ('verbose', 0),
            ('eigensolver', None),
            ('poissonsolver', None),
            ('communicator', mpi.world),
            ('idiotproof', True),
            ('mode', 'fd'),
            ('convergence', {'energy': 0.0005,  # eV / electron
                             'density': 1.0e-4,
                             'eigenstates': 4.0e-8,  # eV^2
                             'bands': 'occupied',
                             'forces': None}),  # eV / Ang Max
            ('realspace', None),
            ('symmetry', {'point_group': True,
                          'time_reversal': True,
                          'symmorphic': True,
                          'tolerance': 1e-7})])
        dict.update(self, kwargs)

    def __repr__(self):
        dictrepr = dict.__repr__(self)
        repr = 'InputParameters(**%s)' % dictrepr
        return repr
    
    def __getattr__(self, key):
        return self[key]

    def __setattr__(self, key, value):
        assert key in self
        self[key] = value

    def update(self, parameters):
        for key, value in parameters.items():
            assert key in self
        dict.update(self, parameters)

    def read(self, reader):
        """Read state from file."""

        r = reader

        version = r['version']
        
        assert version >= 0.3
    
        self.xc = r['XCFunctional']
        self.nbands = r.dimension('nbands')
        self.spinpol = (r.dimension('nspins') == 2)

        bzk_kc = r.get('BZKPoints', broadcast=True)
        if r.has_array('NBZKPoints'):
            self.kpts = r.get('NBZKPoints', broadcast=True)
            if r.has_array('MonkhorstPackOffset'):
                offset_c = r.get('MonkhorstPackOffset', broadcast=True)
                if offset_c.any():
                    self.kpts = monkhorst_pack(self.kpts) + offset_c
        else:
            self.kpts = bzk_kc

        if version < 4:
            self.symmetry = usesymm2symmetry(r['UseSymmetry'])
        else:
            self.symmetry = {'point_group': r['SymmetryOnSwitch'],
                             'symmorphic': r['SymmetrySymmorphicSwitch'],
                             'time_reversal': r['SymmetryTimeReversalSwitch'],
                             'tolerance': r['SymmetryToleranceCriterion']}

        try:
            self.basis = r['BasisSet']
        except KeyError:
            pass

        if version >= 2:
            try:
                h = r['GridSpacing']
            except KeyError:  # CMR can't handle None!
                h = None
            if h is not None:
                self.h = Bohr * h
            if r.has_array('GridPoints'):
                self.gpts = r.get('GridPoints')
        else:
            if version >= 0.9:
                h = r['GridSpacing']
            else:
                h = None

            gpts = ((r.dimension('ngptsx') + 1) // 2 * 2,
                    (r.dimension('ngptsy') + 1) // 2 * 2,
                    (r.dimension('ngptsz') + 1) // 2 * 2)

            if h is None:
                self.gpts = gpts
            else:
                self.h = Bohr * h

        self.lmax = r['MaximumAngularMomentum']
        self.setups = r['SetupTypes']
        self.fixdensity = r['FixDensity']
        if version <= 0.4:
            # Old version: XXX
            print(('# Warning: Reading old version 0.3/0.4 restart files ' +
                  'will be disabled some day in the future!'))
            self.convergence['eigenstates'] = r['Tolerance']
        else:
            nbtc = r['NumberOfBandsToConverge']
            if not isinstance(nbtc, (int, str)):
                # The string 'all' was eval'ed to the all() function!
                nbtc = 'all'
            if version < 5:
                force_crit = None
            else:
                force_crit = r['ForcesConvergenceCriterion']
                if force_crit is not None:
                    force_crit *= (Hartree / Bohr)
            self.convergence = {'density': r['DensityConvergenceCriterion'],
                                'energy':
                                r['EnergyConvergenceCriterion'] * Hartree,
                                'eigenstates':
                                r['EigenstatesConvergenceCriterion'],
                                'bands': nbtc,
                                'forces': force_crit}

            if version < 1:
                # Volume per grid-point:
                dv = (abs(np.linalg.det(r.get('UnitCell'))) /
                      (gpts[0] * gpts[1] * gpts[2]))
                self.convergence['eigenstates'] *= Hartree**2 * dv

            if version <= 0.6:
                mixer = 'Mixer'
                weight = r['MixMetric']
            elif version <= 0.7:
                mixer = r['MixClass']
                weight = r['MixWeight']
                metric = r['MixMetric']
                if metric is None:
                    weight = 1.0
            else:
                mixer = r['MixClass']
                weight = r['MixWeight']

            if mixer == 'Mixer':
                from gpaw.mixer import Mixer
            elif mixer == 'MixerSum':
                from gpaw.mixer import MixerSum as Mixer
            elif mixer == 'MixerSum2':
                from gpaw.mixer import MixerSum2 as Mixer
            elif mixer == 'MixerDif':
                from gpaw.mixer import MixerDif as Mixer
            elif mixer == 'DummyMixer':
                from gpaw.mixer import DummyMixer as Mixer
            else:
                Mixer = None

            if Mixer is None:
                self.mixer = None
            else:
                self.mixer = Mixer(r['MixBeta'], r['MixOld'], weight)
            
        if version == 0.3:
            # Old version: XXX
            print(('# Warning: Reading old version 0.3 restart files is ' +
                  'dangerous and will be disabled some day in the future!'))
            self.stencils = (2, 3)
            self.charge = 0.0
            fixmom = False
        else:
            self.stencils = (r['KohnShamStencil'],
                             r['InterpolationStencil'])
            if r['PoissonStencil'] == 999:
                self.poissonsolver = FFTPoissonSolver()
            else:
                self.poissonsolver = PoissonSolver(nn=r['PoissonStencil'])
            self.charge = r['Charge']
            fixmom = r['FixMagneticMoment']

        self.occupations = FermiDirac(r['FermiWidth'] * Hartree,
                                      fixmagmom=fixmom)

        try:
            self.mode = r['Mode']
        except KeyError:
            self.mode = 'fd'

        if self.mode == 'pw':
            self.mode = PW(ecut=r['PlaneWaveCutoff'] * Hartree)
            
        if len(bzk_kc) == 1 and not bzk_kc[0].any():
            # Gamma point only:
            if r['DataType'] == 'Complex':
                self.dtype = complex
                
        try:
            from gpaw.external import dict2potential
            self.external = dict2potential(r['ExternalPotential'])
        except KeyError:
            pass



from gpaw.tddft import photoabsorption_spectrum
from ase import Atoms
from gpaw import GPAW
from gpaw.lcaotddft import LCAOTDDFT
from gpaw.mpi import world

xc = 'oldLDA'
c = +1
h = 0.4
b = 'dzp'
sy = 'Na2'
positions = [[0.00,0.00,0.00],[0.00,0.00,2.00]]
atoms = Atoms(symbols=sy, positions = positions)
atoms.center(vacuum=3)

# LCAO-RT-TDDFT
calc = GPAW(mode='lcao', nbands=1, xc=xc, h=h, basis=b, dtype=complex, 
            charge=c, width=0, convergence={'density':1e-8}, setups={'Na': '1'})
atoms.set_calculator(calc)
atoms.get_potential_energy()
calc.write('Na2.gpw','all')
del calc

calc = LCAOTDDFT('Na2.gpw')
dmfile = sy+'_lcao_restart_'+b+'_rt_z.dm'+str(world.size)
specfile = sy+'_lcao_restart_'+b+'_rt_z.spectrum'+str(world.size)
calc.absorption_kick([0.0,0,0.001])
calc.propagate(10, 20, dmfile)
if world.rank == 0:
    photoabsorption_spectrum(dmfile, specfile)




# -*- coding: utf-8
# In an attempt to appease epydoc and still have readable docstrings,
# the vertical bar | is represented by u'\u2758' in this module.
"""This module calculates XC kernels for the response model.
"""
from __future__ import print_function

import numpy as np
from gpaw.xc import XC
from gpaw.sphere.lebedev import weight_n, R_nv
from gpaw.mpi import world, rank, size
from gpaw.io.tar import Reader
from ase.utils.timing import Timer
from ase.units import Bohr, Ha


def get_xc_kernel(pd, chi0, functional='ALDA', chi0_wGG=None):
    """Factory function that calls the relevant functions below"""

    calc = chi0.calc
    fd = chi0.fd
    nspins = len(calc.density.nt_sG)
    assert nspins == 1

    if functional[0] == 'A':
        # Standard adiabatic kernel
        print('Calculating %s kernel' % functional, file=fd)
        Kxc_sGG = calculate_Kxc(pd, calc, functional=functional)
    elif functional[0] == 'r':
        # Renormalized kernel
        print('Calculating %s kernel' % functional, file=fd)
        from gpaw.xc.fxc import KernelDens
        kernel = KernelDens(calc,
                            functional,
                            [pd.kd.bzk_kc[0]],
                            fd,
                            calc.wfs.kd.N_c,
                            None,
                            ecut=pd.ecut * Ha,
                            tag='',
                            timer=Timer(),
                            )
        kernel.calculate_fhxc()
        r = Reader('fhxc_%s_%s_%s_%s.gpw' %
                   ('', functional, pd.ecut * Ha, 0))
        Kxc_sGG = np.array([r.get('fhxc_sGsG')])
        
        v_G = 4 * np.pi / pd.G2_qG[0]
        Kxc_sGG[0] -= np.diagflat(v_G)

        if pd.kd.gamma:
            Kxc_sGG[:, 0, :] = 0.0
            Kxc_sGG[:, :, 0] = 0.0
    elif functional[:2] == 'LR':
        print('Calculating LR kernel with alpha = %s' % functional[2:],
              file=fd)
        Kxc_sGG = calculate_lr_kernel(pd, calc,
                                      alpha=float(functional[2:]))
    elif functional == 'DM':
        print('Calculating DM kernel', file=fd)
        Kxc_sGG = calculate_dm_kernel(pd, calc)
    elif functional == 'Bootstrap':
        print('Calculating Bootstrap kernel', file=fd)
        if chi0.world.rank == 0:
            chi0_GG = chi0_wGG[0]
            chi0.world.broadcast(chi0_GG, 0)
        else:
            nG = pd.ngmax
            chi0_GG = np.zeros((nG, nG), complex)
            chi0.world.broadcast(chi0_GG, 0)
        Kxc_sGG = calculate_bootstrap_kernel(pd, chi0_GG, fd)
    else:
        raise ValueError('%s kernel not implemented' % functional)

    return Kxc_sGG


def calculate_Kxc(pd, calc, functional='ALDA', density_cut=None):
    """ALDA kernel"""
    
    gd = pd.gd
    npw = pd.ngmax
    nG = pd.gd.N_c
    vol = pd.gd.volume
    G_Gv = pd.get_reciprocal_vectors()
    nt_sG = calc.density.nt_sG
    R_av = calc.atoms.positions / Bohr
    setups = calc.wfs.setups
    D_asp = calc.density.D_asp

    # The soft part
    # assert np.abs(nt_sG[0].shape - nG).sum() == 0
    if functional == 'ALDA_X':
        x_only = True
        A_x = -3. / 4. * (3. / np.pi)**(1. / 3.)
        nspins = len(nt_sG)
        assert nspins in [1, 2]
        fxc_sg = nspins**(1. / 3.) * 4. / 9. * A_x * nt_sG**(-2. / 3.)
    else:
        assert len(nt_sG) == 1
        x_only = False
        fxc_sg = np.zeros_like(nt_sG)
        xc = XC(functional[1:])
        xc.calculate_fxc(gd, nt_sG, fxc_sg)

    if density_cut is not None:
        fxc_sg[np.where(nt_sG * len(nt_sG) < density_cut)] = 0.0

    # FFT fxc(r)
    nG0 = nG[0] * nG[1] * nG[2]
    tmp_sg = [np.fft.fftn(fxc_sg[s]) * vol / nG0 for s in range(len(nt_sG))]

    Kxc_sGG = np.zeros((len(fxc_sg), npw, npw), dtype=complex)
    for s in range(len(fxc_sg)):
        for iG, iQ in enumerate(pd.Q_qG[0]):
            iQ_c = (np.unravel_index(iQ, nG) + nG // 2) % nG - nG // 2
            for jG, jQ in enumerate(pd.Q_qG[0]):
                jQ_c = (np.unravel_index(jQ, nG) + nG // 2) % nG - nG // 2
                ijQ_c = (iQ_c - jQ_c)
                if (abs(ijQ_c) < nG // 2).all():
                    Kxc_sGG[s, iG, jG] = tmp_sg[s][tuple(ijQ_c)]

    # The PAW part
    KxcPAW_sGG = np.zeros_like(Kxc_sGG)
    dG_GGv = np.zeros((npw, npw, 3))
    for v in range(3):
        dG_GGv[:, :, v] = np.subtract.outer(G_Gv[:, v], G_Gv[:, v])

    for a, setup in enumerate(setups):
        if rank == a % size:
            rgd = setup.xc_correction.rgd
            n_qg = setup.xc_correction.n_qg
            nt_qg = setup.xc_correction.nt_qg
            nc_g = setup.xc_correction.nc_g
            nct_g = setup.xc_correction.nct_g
            Y_nL = setup.xc_correction.Y_nL
            dv_g = rgd.dv_g

            D_sp = D_asp[a]
            B_pqL = setup.xc_correction.B_pqL
            D_sLq = np.inner(D_sp, B_pqL.T)
            nspins = len(D_sp)

            f_sg = rgd.empty(nspins)
            ft_sg = rgd.empty(nspins)

            n_sLg = np.dot(D_sLq, n_qg)
            nt_sLg = np.dot(D_sLq, nt_qg)

            # Add core density
            n_sLg[:, 0] += np.sqrt(4. * np.pi) / nspins * nc_g
            nt_sLg[:, 0] += np.sqrt(4. * np.pi) / nspins * nct_g

            coefatoms_GG = np.exp(-1j * np.inner(dG_GGv, R_av[a]))
            for n, Y_L in enumerate(Y_nL):
                w = weight_n[n]
                f_sg[:] = 0.0
                n_sg = np.dot(Y_L, n_sLg)
                if x_only:
                    f_sg = nspins * (4 / 9.) * A_x * (nspins * n_sg)**(-2 / 3.)
                else:
                    xc.calculate_fxc(rgd, n_sg, f_sg)

                ft_sg[:] = 0.0
                nt_sg = np.dot(Y_L, nt_sLg)
                if x_only:
                    ft_sg = nspins * (4 / 9.) * (A_x
                                                 * (nspins * nt_sg)**(-2 / 3.))
                else:
                    xc.calculate_fxc(rgd, nt_sg, ft_sg)
                for i in range(len(rgd.r_g)):
                    coef_GG = np.exp(-1j * np.inner(dG_GGv, R_nv[n])
                                     * rgd.r_g[i])
                    for s in range(len(f_sg)):
                        KxcPAW_sGG[s] += w * np.dot(coef_GG,
                                                    (f_sg[s, i] -
                                                     ft_sg[s, i])
                                                    * dv_g[i]) * coefatoms_GG

    world.sum(KxcPAW_sGG)
    Kxc_sGG += KxcPAW_sGG

    if pd.kd.gamma:
        Kxc_sGG[:, 0, :] = 0.0
        Kxc_sGG[:, :, 0] = 0.0

    return Kxc_sGG / vol

    
def calculate_lr_kernel(pd, calc, alpha=0.2):
    """Long range kernel: fxc = \alpha / |q+G|^2"""
    
    assert pd.kd.gamma

    f_G = np.zeros(len(pd.G2_qG[0]))
    f_G[0] = -alpha
    f_G[1:] = -alpha / pd.G2_qG[0][1:]

    return np.array([np.diag(f_G)])
    
    
def calculate_dm_kernel(pd, calc):
    """Density matrix kernel"""
    
    assert pd.kd.gamma
    
    nv = calc.wfs.setups.nvalence
    psit_nG = np.array([calc.wfs.kpt_u[0].psit_nG[n]
                        for n in range(4 * nv)])
    vol = np.linalg.det(calc.wfs.gd.cell_cv)
    Ng = np.prod(calc.wfs.gd.N_c)
    rho_GG = np.dot(psit_nG.conj().T, psit_nG) * vol / Ng**2

    maxG2 = np.max(pd.G2_qG[0])
    cut_G = np.arange(calc.wfs.pd.ngmax)[calc.wfs.pd.G2_qG[0] <= maxG2]
    
    G_G = pd.G2_qG[0]**0.5
    G_G[0] = 1.0
    
    Kxc_GG = np.diagflat(4 * np.pi / G_G**2)
    Kxc_GG = np.dot(Kxc_GG, rho_GG.take(cut_G, 0).take(cut_G, 1))
    Kxc_GG -= 4 * np.pi * np.diagflat(1.0 / G_G**2)

    return np.array([Kxc_GG])
    

def calculate_bootstrap_kernel(pd, chi0_GG, fd):
    """Bootstrap kernel PRL 107, 186401"""

    if pd.kd.gamma:
        v_G = np.zeros(len(pd.G2_qG[0]))
        v_G[0] = 4 * np.pi
        v_G[1:] = 4 * np.pi / pd.G2_qG[0][1:]
    else:
        v_G = 4 * np.pi / pd.G2_qG[0]

    nG = len(v_G)
    K_GG = np.diag(v_G)

    fxc_GG = np.zeros((nG, nG), dtype=complex)
    dminv_GG = np.zeros((nG, nG), dtype=complex)
                
    for iscf in range(120):
        dminvold_GG = dminv_GG.copy()
        Kxc_GG = K_GG + fxc_GG

        chi_GG = np.dot(np.linalg.inv(np.eye(nG, nG)
                                      - np.dot(chi0_GG, Kxc_GG)), chi0_GG)
        dminv_GG = np.eye(nG, nG) + np.dot(K_GG, chi_GG)

        alpha = dminv_GG[0, 0] / (K_GG[0, 0] * chi0_GG[0, 0])
        fxc_GG = alpha * K_GG
        print(iscf, 'alpha =', alpha, file=fd)
        error = np.abs(dminvold_GG - dminv_GG).sum()
        if np.sum(error) < 0.1:
            print('Self consistent fxc finished in %d iterations !' % iscf,
                  file=fd)
            break
        if iscf > 100:
            print('Too many fxc scf steps !', file=fd)
    
    return np.array([fxc_GG])



import numpy as np
import os
from gpaw.io.hdf5_highlevel import File
from gpaw.mpi import world, rank

# write a slice of NumPy array
data = np.arange(10, dtype=float)
sub = data[::2]

if world.size > 1:
    comm = world.get_c_object()
else:
    comm = None

file = File('tmp.hdf5', 'w', comm=comm)

dset = file.create_dataset('noncont', sub.shape, sub.dtype) 
if rank == 0:
    selection = 'all'
else:
    selection = None
dset.write(sub, selection)
dset.close()
file.close()

# read data back
file = File('tmp.hdf5', 'r', comm=comm)
dset = file['noncont']
new_data = np.ndarray(dset.shape, dset.dtype, order='C')
dset.read(new_data)
assert((new_data == sub).all)
# read also to noncontiguous array
new_data = np.zeros(10, float)
new_sub = new_data[::2]
dset.read(new_sub)
assert((new_sub == sub).all)

dset.close()
file.close()
if rank == 0:
    os.remove('tmp.hdf5')



import os
import platform
import sys


def get_mpi_implementation():
    mpi = os.environ.get('GPAW_MPI_IMPLEMENTATION')
    if mpi is not None:
        return mpi
    
    machine = platform.uname()[4]
    
    if machine == 'sun4u':
        return 'sun'

    if sys.platform == 'aix5':
        return 'poe'

    if sys.platform == 'ia64':
        return 'mpich'
                
    output = os.popen('mpicc --showme 2> /dev/null', 'r').read()
    if output != '':
        if 'openmpi' in output:
            return 'openmpi'
        else:
            return 'lam'

    output = os.popen('mpicc -show 2> /dev/null', 'r').read()
    if output != '':
        if 'mvapich' in output:
            return 'mvapich'
        else:
            return 'mpich'

    return ''


def get_mpi_command(debug=False):
    cmd = os.environ.get('GPAW_MPI_COMMAND')
    if cmd is not None:
        return cmd

    mpi = get_mpi_implementation()

    if mpi == 'sun':
        return 'mprun -np %(np)d %(job)s &'

    if mpi == 'poe':
        if 'LOADL_PROCESSOR_LIST' in os.environ:
            return "poe '%(job)s' &"
        else:
            return "poe '%(job)s' -procs %(np)d -hfile %(hostfile)s &"
                
    if mpi == 'mpich':
        output = os.popen('mpiexec', 'r').read()
        if output != '':
            return 'mpiexec -n %(np)d %(job)s'
        output = os.popen('mpirun', 'r').read()
        if output != '':
            return 'mpirun -n %(np)d %(job)s'
        raise NotImplementedError

    if mpi == 'mvapich':
        raise NotImplementedError

    if mpi == 'lam':
        return ('lamnodes; '
                'if [ $? != 0 ]; then '
                '  lamboot -H %(hostfile)s; '
                'fi; '
                'mpirun -np %(np)d %(job)s &')
    
    if mpi == 'openmpi':
        return 'mpirun -np %(np)d --hostfile %(hostfile)s %(job)s &'

    raise NotImplementedError


if __name__ == '__main__':
    print(get_mpi_implementation())
    print(get_mpi_command(debug=False))
    print(get_mpi_command(debug=True))




# Copyright (C) 2003  CAMP
# Please see the accompanying LICENSE file for further information.

from math import pi
import warnings

import numpy as np
from numpy.fft import fftn, ifftn, fft2, ifft2

from gpaw.transformers import Transformer
from gpaw.fd_operators import Laplace, LaplaceA, LaplaceB
from gpaw import PoissonConvergenceError
from gpaw.utilities.blas import axpy
from gpaw.utilities.gauss import Gaussian
from gpaw.utilities.ewald import madelung
from gpaw.utilities.grid_redistribute import GridRedistributor
from gpaw.utilities.tools import construct_reciprocal
import _gpaw


POISSON_GRID_WARNING="""Grid unsuitable for Poisson solver!

The Poisson solver does not have sufficient multigrid levels for good
performance and will converge inefficiently if at all, or yield wrong
results.

You may need to manually specify a grid such that the number of points
along each direction is divisible by a high power of 2, such as 8, 16,
or 32 depending on system size; examples:

  GPAW(gpts=(32, 32, 288))

or

  from gpaw.utilities import h2gpts
  GPAW(gpts=h2gpts(0.2, atoms.get_cell(), idiv=16))

Parallelizing over very small domains can also undesirably limit the
number of multigrid levels even if the total number of grid points
is divisible by a high power of 2."""


class PoissonSolver:
    def __init__(self, nn=3, relax='J', eps=2e-10, maxiter=1000,
                 remove_moment=None, use_charge_center=False):
        self.relax = relax
        self.nn = nn
        self.eps = eps
        self.charged_periodic_correction = None
        self.maxiter = maxiter
        self.remove_moment = remove_moment
        self.use_charge_center = use_charge_center

        # Relaxation method
        if relax == 'GS':
            # Gauss-Seidel
            self.relax_method = 1
        elif relax == 'J':
            # Jacobi
            self.relax_method = 2
        else:
            raise NotImplementedError('Relaxation method %s' % relax)

        self.description = None

    def get_stencil(self):
        return self.nn

    def set_grid_descriptor(self, gd):
        # Should probably be renamed initialize
        self.gd = gd
        scale = -0.25 / pi

        if self.nn == 'M':
            if not gd.orthogonal:
                raise RuntimeError('Cannot use Mehrstellen stencil with '
                                   'non orthogonal cell.')

            self.operators = [LaplaceA(gd, -scale)]
            self.B = LaplaceB(gd)
        else:
            self.operators = [Laplace(gd, scale, self.nn)]
            self.B = None

        self.interpolators = []
        self.restrictors = []

        level = 0
        self.presmooths = [2]
        self.postsmooths = [1]

        # Weights for the relaxation,
        # only used if 'J' (Jacobi) is chosen as method
        self.weights = [2.0 / 3.0]

        while level < 8:
            try:
                gd2 = gd.coarsen()
            except ValueError:
                break
            self.operators.append(Laplace(gd2, scale, 1))
            self.interpolators.append(Transformer(gd2, gd))
            self.restrictors.append(Transformer(gd, gd2))
            self.presmooths.append(4)
            self.postsmooths.append(4)
            self.weights.append(1.0)
            level += 1
            gd = gd2

        self.levels = level

        if self.operators[-1].gd.N_c.max() > 36:
            # Try to warn exactly once no matter how one uses the solver.
            if gd.comm.parent is None:
                warn = (gd.comm.rank == 0)
            else:
                warn = (gd.comm.parent.rank == 0)

            if warn:
                warntxt = '\n'.join([POISSON_GRID_WARNING, '',
                                     self.get_description()])
            else:
                warntxt = ('Poisson warning from domain rank %d'
                           % self.gd.comm.rank)

            # Warn from all ranks to avoid deadlocks.
            warnings.warn(warntxt, stacklevel=2)

    def get_description(self):
        name = {1: 'Gauss-Seidel', 2: 'Jacobi'}[self.relax_method]
        coarsest_grid = self.operators[-1].gd.N_c
        coarsest_grid_string = ' x '.join([str(N) for N in coarsest_grid])
        assert self.levels + 1 == len(self.operators)
        lines = ['%s solver with %d multi-grid levels'
                 % (name, self.levels + 1),
                 '    Coarsest grid: %s points' % coarsest_grid_string]
        if coarsest_grid.max() > 24:
            # This friendly warning has lower threshold than the big long
            # one that we print when things are really bad.
            lines.extend(['    Warning: Coarse grid has more than 24 points.',
                          '             More multi-grid levels recommended.'])
        lines.extend(['    Stencil: %s' % self.operators[0].description,
                      '    Tolerance: %e' % self.eps,
                      '    Max iterations: %d' % self.maxiter])
        if self.remove_moment is not None:
            lines.append('    Remove moments up to L=%d' % self.remove_moment)
        if self.use_charge_center:
            lines.append('    Compensate for charged system using center of '
                         'majority charge')
        return '\n'.join(lines)

    def initialize(self, load_gauss=False):
        # Should probably be renamed allocate
        gd = self.gd
        self.rhos = [gd.empty()]
        self.phis = [None]
        self.residuals = [gd.empty()]
        for level in range(self.levels):
            gd2 = gd.coarsen()
            self.phis.append(gd2.empty())
            self.rhos.append(gd2.empty())
            self.residuals.append(gd2.empty())
            gd = gd2
        assert len(self.phis) == len(self.rhos)
        level += 1
        assert level == self.levels

        self.step = 0.66666666 / self.operators[0].get_diagonal_element()
        self.presmooths[level] = 8
        self.postsmooths[level] = 8

        if load_gauss:
            self.load_gauss()

    def load_gauss(self, center=None):
        if not hasattr(self, 'rho_gauss') or center is not None:
            gauss = Gaussian(self.gd, center=center)
            self.rho_gauss = gauss.get_gauss(0)
            self.phi_gauss = gauss.get_gauss_pot(0)

    def solve(self, phi, rho, charge=None, eps=None, maxcharge=1e-6,
              zero_initial_phi=False):
        assert np.all(phi.shape == self.gd.n_c)
        assert np.all(rho.shape == self.gd.n_c)

        if eps is None:
            eps = self.eps
        actual_charge = self.gd.integrate(rho)
        background = (actual_charge / self.gd.dv /
                      self.gd.get_size_of_global_array().prod())

        if self.remove_moment:
            assert not self.gd.pbc_c.any()
            if not hasattr(self, 'gauss'):
                self.gauss = Gaussian(self.gd)
            rho_neutral = rho.copy()
            phi_cor_L = []
            for L in range(self.remove_moment):
                phi_cor_L.append(self.gauss.remove_moment(rho_neutral, L))
            # Remove multipoles for better initial guess
            for phi_cor in phi_cor_L:
                phi -= phi_cor

            niter = self.solve_neutral(phi, rho_neutral, eps=eps)
            # correct error introduced by removing multipoles
            for phi_cor in phi_cor_L:
                phi += phi_cor

            return niter
        if charge is None:
            charge = actual_charge
        if abs(charge) <= maxcharge:
            # System is charge neutral. Use standard solver
            return self.solve_neutral(phi, rho - background, eps=eps)

        elif abs(charge) > maxcharge and self.gd.pbc_c.all():
            # System is charged and periodic. Subtract a homogeneous
            # background charge

            # Set initial guess for potential
            if zero_initial_phi:
                phi[:] = 0.0

            iters = self.solve_neutral(phi, rho - background, eps=eps)
            return iters

        elif abs(charge) > maxcharge and not self.gd.pbc_c.any():
            # The system is charged and in a non-periodic unit cell.
            # Determine the potential by 1) subtract a gaussian from the
            # density, 2) determine potential from the neutralized density
            # and 3) add the potential from the gaussian density.

            # Load necessary attributes

            # use_charge_center: The monopole will be removed at the
            # center of the majority charge, which prevents artificial
            # dipoles.
            # Due to the shape of the Gaussian and it's Fourier-Transform,
            # the Gaussian representing the charge should stay at least
            # 7 gpts from the borders - see:
            # https://listserv.fysik.dtu.dk/pipermail/gpaw-developers/2015-July/005806.html
            if self.use_charge_center:
                charge_sign = actual_charge / abs(actual_charge)
                rho_sign = rho * charge_sign
                rho_sign[np.where(rho_sign < 0)] = 0
                absolute_charge = self.gd.integrate(rho_sign)
                center = - self.gd.calculate_dipole_moment(rho_sign) \
                        / absolute_charge
                border_offset = np.inner(self.gd.h_cv, np.array((7, 7, 7)))
                borders = np.inner(self.gd.h_cv, self.gd.N_c)
                borders -= border_offset
                if np.any(center > borders) or np.any(center < border_offset):
                    raise RuntimeError(
                            'Poisson solver: center of charge outside' + \
                            ' borders - please increase box')
                    center[np.where(center > borders)] = borders
                self.load_gauss(center=center)
            else:
                self.load_gauss()

            # Remove monopole moment
            q = actual_charge / np.sqrt(4 * pi)  # Monopole moment
            rho_neutral = rho - q * self.rho_gauss  # neutralized density

            # Set initial guess for potential
            if zero_initial_phi:
                phi[:] = 0.0
            else:
                axpy(-q, self.phi_gauss, phi)  # phi -= q * self.phi_gauss

            # Determine potential from neutral density using standard solver
            niter = self.solve_neutral(phi, rho_neutral, eps=eps)

            # correct error introduced by removing monopole
            axpy(q, self.phi_gauss, phi)  # phi += q * self.phi_gauss

            return niter
        else:
            # System is charged with mixed boundaryconditions
            msg = ('Charged systems with mixed periodic/zero'
                   ' boundary conditions')
            raise NotImplementedError(msg)

    def solve_neutral(self, phi, rho, eps=2e-10):
        self.phis[0] = phi

        if self.B is None:
            self.rhos[0][:] = rho
        else:
            self.B.apply(rho, self.rhos[0])

        niter = 1
        maxiter = self.maxiter
        while self.iterate2(self.step) > eps and niter < maxiter:
            niter += 1
        if niter == maxiter:
            msg = 'Poisson solver did not converge in %d iterations!' % maxiter
            raise PoissonConvergenceError(msg)

        # Set the average potential to zero in periodic systems
        if np.alltrue(self.gd.pbc_c):
            phi_ave = self.gd.comm.sum(np.sum(phi.ravel()))
            N_c = self.gd.get_size_of_global_array()
            phi_ave /= np.product(N_c)
            phi -= phi_ave

        return niter

    def iterate(self, step, level=0):
        residual = self.residuals[level]
        niter = 0
        while True:
            niter += 1
            if level > 0 and niter == 1:
                residual[:] = -self.rhos[level]
            else:
                self.operators[level].apply(self.phis[level], residual)
                residual -= self.rhos[level]
            error = self.gd.comm.sum(np.vdot(residual, residual))
            if niter == 1 and level < self.levels:
                self.restrictors[level].apply(residual, self.rhos[level + 1])
                self.phis[level + 1][:] = 0.0
                self.iterate(4.0 * step, level + 1)
                self.interpolators[level].apply(self.phis[level + 1], residual)
                self.phis[level] -= residual
                continue
            residual *= step
            self.phis[level] -= residual
            if niter == 2:
                break

        return error

    def iterate2(self, step, level=0):
        """Smooths the solution in every multigrid level"""

        residual = self.residuals[level]

        if level < self.levels:
            self.operators[level].relax(self.relax_method,
                                        self.phis[level],
                                        self.rhos[level],
                                        self.presmooths[level],
                                        self.weights[level])

            self.operators[level].apply(self.phis[level], residual)
            residual -= self.rhos[level]
            self.restrictors[level].apply(residual,
                                          self.rhos[level + 1])
            self.phis[level + 1][:] = 0.0
            self.iterate2(4.0 * step, level + 1)
            self.interpolators[level].apply(self.phis[level + 1], residual)
            self.phis[level] -= residual

        self.operators[level].relax(self.relax_method,
                                    self.phis[level],
                                    self.rhos[level],
                                    self.postsmooths[level],
                                    self.weights[level])
        if level == 0:
            self.operators[level].apply(self.phis[level], residual)
            residual -= self.rhos[level]
            error = self.gd.comm.sum(np.dot(residual.ravel(),
                                            residual.ravel())) * self.gd.dv
            return error

    def estimate_memory(self, mem):
        # XXX Memory estimate works only for J and GS, not FFT solver
        # Poisson solver appears to use same amount of memory regardless
        # of whether it's J or GS, which is a bit strange

        gdbytes = self.gd.bytecount()
        nbytes = -gdbytes  # No phi on finest grid, compensate ahead
        for level in range(self.levels):
            nbytes += 3 * gdbytes  # Arrays: rho, phi, residual
            gdbytes //= 8
        mem.subnode('rho, phi, residual [%d levels]' % self.levels, nbytes)

    def __repr__(self):
        template = 'PoissonSolver(relax=\'%s\', nn=%s, eps=%e)'
        representation = template % (self.relax, repr(self.nn), self.eps)
        return representation


class NoInteractionPoissonSolver:
    relax_method = 0
    nn = 1

    def get_description(self):
        return 'No interaction'

    def get_stencil(self):
        return 1

    def solve(self, phi, rho, charge):
        return 0

    def set_grid_descriptor(self, gd):
        pass

    def initialize(self):
        pass


class FFTPoissonSolver(PoissonSolver):
    """FFT Poisson solver for general unit cells."""

    relax_method = 0
    nn = 999

    def __init__(self, eps=2e-10):
        # This class inherits from PoissonSolver but it has almost none of
        # its capabilities and doesn't even invoke its constructor!!
        self.charged_periodic_correction = None
        self.remove_moment = None
        self.eps = eps

    def get_description(self):
        return 'FFT'

    def set_grid_descriptor(self, gd):
        assert gd.pbc_c.all()
        self.gd = gd

    def initialize(self):
        if self.gd.comm.rank == 0:
            self.k2_Q, self.N3 = construct_reciprocal(self.gd)

    def solve_neutral(self, phi_g, rho_g, eps=None):
        if self.gd.comm.size == 1:
            phi_g[:] = ifftn(fftn(rho_g) * 4.0 * pi / self.k2_Q).real
        else:
            rho_g = self.gd.collect(rho_g)
            if self.gd.comm.rank == 0:
                globalphi_g = ifftn(fftn(rho_g) * 4.0 * pi / self.k2_Q).real
            else:
                globalphi_g = None
            self.gd.distribute(globalphi_g, phi_g)
        return 1

    def estimate_memory(self, mem):
        mem.subnode('(Varies)', 0.0)


class ParallelFFTPoissonSolver(PoissonSolver):
    """FFT Poisson solver for general unit cells."""
    # XXX it is criminally outrageous that this inherits from PoissonSolver!

    relax_method = 0
    nn = 999

    def __init__(self, eps=0.0):
        self.charged_periodic_correction = None
        self.remove_moment = None
        self.eps = eps

    def get_description(self):
        return 'Parallel FFT'

    def set_grid_descriptor(self, gd):
        # We will probably want to use this on non-periodic grids too...
        assert gd.pbc_c.all()
        self.gd = gd
        self.transp_x_yz_1 = GridRedistributor(self.gd, 1, 2)
        self.transp_1_yz_x = GridRedistributor(self.transp_x_yz_1.gd2, 2, 0)
        self.transp_yz_1_x = GridRedistributor(self.transp_1_yz_x.gd2, 0, 1)

    def initialize(self):
        gd = self.transp_yz_1_x.gd2
        k2_Q, N3 = construct_reciprocal(gd, distributed=True)
        self.poisson_factor_Q = 4.0 * np.pi / k2_Q

    def solve_neutral(self, phi_g, rho_g, eps=None):
        # Will be a bit more efficient if reduced dimension is always
        # contiguous.  Probably more things can be improved...
        work = fftn(self.transp_x_yz_1.forth(rho_g), axes=[2])
        work = fftn(self.transp_1_yz_x.forth(work), axes=[0])
        work = fftn(self.transp_yz_1_x.forth(work), axes=[1])
        work *= self.poisson_factor_Q
        work = self.transp_yz_1_x.back(ifftn(work, axes=[1]))
        work = self.transp_1_yz_x.back(ifftn(work, axes=[0]))
        work = self.transp_x_yz_1.back(ifftn(work, axes=[2]).real)
        phi_g[:] = work
        return 1

    def estimate_memory(self, mem):
        mem.subnode('k squared', self.transp_yz_1_x.gd2.bytecount())


class FixedBoundaryPoissonSolver(PoissonSolver):
    """Solve the Poisson equation with FFT in two directions,
    and with central differential method in the third direction."""

    def __init__(self, nn=1):
        # XXX How can this work when it does not call __init___
        # on PoissonSolver? -askhl
        self.nn = nn
        self.charged_periodic_correction = None
        assert self.nn == 1

    def set_grid_descriptor(self, gd):
        assert gd.pbc_c.all()
        assert gd.orthogonal
        self.gd = gd

    def initialize(self, b_phi1, b_phi2):
        distribution = np.zeros([self.gd.comm.size], int)
        if self.gd.comm.rank == 0:
            gd = self.gd
            N_c1 = gd.N_c[:2, np.newaxis]
            i_cq = np.indices(gd.N_c[:2]).reshape((2, -1))
            i_cq += N_c1 // 2
            i_cq %= N_c1
            i_cq -= N_c1 // 2
            B_vc = 2.0 * np.pi * gd.icell_cv.T[:2, :2]
            k_vq = np.dot(B_vc, i_cq)
            k_vq *= k_vq
            k_vq2 = np.sum(k_vq, axis=0)
            k_vq2 = k_vq2.reshape(-1)

            b_phi1 = fft2(b_phi1, None, (0, 1))
            b_phi2 = fft2(b_phi2, None, (0, 1))

            b_phi1 = b_phi1[:, :, -1].reshape(-1)
            b_phi2 = b_phi2[:, :, 0].reshape(-1)

            loc_b_phi1 = np.array_split(b_phi1, self.gd.comm.size)
            loc_b_phi2 = np.array_split(b_phi2, self.gd.comm.size)
            loc_k_vq2 = np.array_split(k_vq2, self.gd.comm.size)

            self.loc_b_phi1 = loc_b_phi1[0]
            self.loc_b_phi2 = loc_b_phi2[0]
            self.k_vq2 = loc_k_vq2[0]

            for i in range(self.gd.comm.size):
                distribution[i] = len(loc_b_phi1[i])
            self.gd.comm.broadcast(distribution, 0)

            for i in range(1, self.gd.comm.size):
                self.gd.comm.ssend(loc_b_phi1[i], i, 135)
                self.gd.comm.ssend(loc_b_phi2[i], i, 246)
                self.gd.comm.ssend(loc_k_vq2[i], i, 169)
        else:
            self.gd.comm.broadcast(distribution, 0)
            self.loc_b_phi1 = np.zeros([distribution[self.gd.comm.rank]],
                                       dtype=complex)
            self.loc_b_phi2 = np.zeros([distribution[self.gd.comm.rank]],
                                       dtype=complex)
            self.k_vq2 = np.zeros([distribution[self.gd.comm.rank]])
            self.gd.comm.receive(self.loc_b_phi1, 0, 135)
            self.gd.comm.receive(self.loc_b_phi2, 0, 246)
            self.gd.comm.receive(self.k_vq2, 0, 169)

        k_distribution = np.arange(np.sum(distribution))
        self.k_distribution = np.array_split(k_distribution,
                                             self.gd.comm.size)

        self.d1, self.d2, self.d3 = self.gd.N_c
        self.r_distribution = np.array_split(np.arange(self.d3),
                                             self.gd.comm.size)
        self.comm_reshape = not (self.gd.parsize_c[0] == 1
                                 and self.gd.parsize_c[1] == 1)

    def solve(self, phi_g, rho_g, charge=None):
        if charge is None:
            actual_charge = self.gd.integrate(rho_g)
        else:
            actual_charge = charge

        if self.charged_periodic_correction is None:
            self.charged_periodic_correction = madelung(self.gd.cell_cv)

        background = (actual_charge / self.gd.dv /
                      self.gd.get_size_of_global_array().prod())

        self.solve_neutral(phi_g, rho_g - background)
        phi_g += actual_charge * self.charged_periodic_correction

    def scatter_r_distribution(self, global_rho_g, dtype=float):
        d1 = self.d1
        d2 = self.d2
        comm = self.gd.comm
        index = self.r_distribution[comm.rank]
        if comm.rank == 0:
            rho_g1 = global_rho_g[:, :, index]
            for i in range(1, comm.size):
                ind = self.r_distribution[i]
                comm.ssend(global_rho_g[:, :, ind].copy(), i, 178)
        else:
            rho_g1 = np.zeros([d1, d2, len(index)], dtype=dtype)
            comm.receive(rho_g1, 0, 178)
        return rho_g1

    def gather_r_distribution(self, rho_g, dtype=complex):
        comm = self.gd.comm
        index = self.r_distribution[comm.rank]
        d1, d2, d3 = self.d1, self.d2, self.d3
        if comm.rank == 0:
            global_rho_g = np.zeros([d1, d2, d3], dtype)
            global_rho_g[:, :, index] = rho_g
            for i in range(1, comm.size):
                ind = self.r_distribution[i]
                rho_gi = np.zeros([d1, d2, len(ind)], dtype)
                comm.receive(rho_gi, i, 368)
                global_rho_g[:, :, ind] = rho_gi
        else:
            comm.ssend(rho_g, 0, 368)
            global_rho_g = None
        return global_rho_g

    def scatter_k_distribution(self, global_rho_g):
        comm = self.gd.comm
        index = self.k_distribution[comm.rank]
        if comm.rank == 0:
            rho_g = global_rho_g[index]
            for i in range(1, comm.size):
                ind = self.k_distribution[i]
                comm.ssend(global_rho_g[ind], i, 370)
        else:
            rho_g = np.zeros([len(index), self.d3], dtype=complex)
            comm.receive(rho_g, 0, 370)
        return rho_g

    def gather_k_distribution(self, phi_g):
        comm = self.gd.comm
        index = self.k_distribution[comm.rank]
        d12 = self.d1 * self.d2
        if comm.rank == 0:
            global_phi_g = np.zeros([d12, self.d3], dtype=complex)
            global_phi_g[index] = phi_g
            for i in range(1, comm.size):
                ind = self.k_distribution[i]
                phi_gi = np.zeros([len(ind), self.d3], dtype=complex)
                comm.receive(phi_gi, i, 569)
                global_phi_g[ind] = phi_gi
        else:
            comm.ssend(phi_g, 0, 569)
            global_phi_g = None
        return global_phi_g

    def solve_neutral(self, phi_g, rho_g):
        # b_phi1 and b_phi2 are the boundary Hartree potential values
        # of left and right sides

        if self.comm_reshape:
            global_rho_g0 = self.gd.collect(rho_g)
            rho_g1 = self.scatter_r_distribution(global_rho_g0)
        else:
            rho_g1 = rho_g

        # use copy() to avoid the C_contiguous=False
        rho_g2 = fft2(rho_g1, None, (0, 1)).copy()

        global_rho_g = self.gather_r_distribution(rho_g2)
        if self.gd.comm.rank == 0:
            global_rho_g.shape = (self.d1 * self.d2, self.d3)
        rho_g3 = self.scatter_k_distribution(global_rho_g)

        du0 = np.zeros(self.d3 - 1, dtype=complex)
        du20 = np.zeros(self.d3 - 2, dtype=complex)
        h2 = self.gd.h_cv[2, 2] ** 2

        phi_g1 = np.zeros(rho_g3.shape, dtype=complex)
        index = self.k_distribution[self.gd.comm.rank]
        for phi, rho, rv2, bp1, bp2, i in zip(phi_g1, rho_g3,
                                              self.k_vq2,
                                              self.loc_b_phi1,
                                              self.loc_b_phi2,
                                              range(len(index))):
            A = np.zeros(self.d3, dtype=complex) + 2 + h2 * rv2
            phi = rho * np.pi * 4 * h2
            phi[0] += bp1
            phi[-1] += bp2
            du = du0 - 1
            dl = du0 - 1
            du2 = du20 - 1
            _gpaw.linear_solve_tridiag(self.d3, A, du, dl, du2, phi)
            phi_g1[i] = phi

        global_phi_g = self.gather_k_distribution(phi_g1)
        if self.gd.comm.rank == 0:
            global_phi_g.shape = (self.d1, self.d2, self.d3)
        phi_g2 = self.scatter_r_distribution(global_phi_g, dtype=complex)
        # use copy() to avoid the C_contiguous=False
        phi_g3 = ifft2(phi_g2, None, (0, 1)).real.copy()
        if self.comm_reshape:
            global_phi_g = self.gather_r_distribution(phi_g3, dtype=float)
            self.gd.distribute(global_phi_g, phi_g)
        else:
            phi_g[:] = phi_g3



import copy
import pickle
import numpy as np

from ase.units import Hartree

from gpaw.mpi import world, send_string, receive_string, broadcast_string
from gpaw.utilities.blas import gemm


def tw(mat, filename):
    fd = open(filename, 'wb')
    pickle.dump(mat, fd, 2)
    fd.close()

    
def tr(filename):
    fd = open(filename, 'rb')
    mat = pickle.load(fd)
    fd.close()
    return mat

    
def write(filename, name, data, dimension, dtype=float):
    import gpaw.io.tar as io
    if world.rank == 0:
        w = io.Writer(filename)
        dim = ()
        for i in range(len(dimension)):
            w.dimension(str(i), dimension[i])
            dim += (str(i),)
        w.add(name, dim, dtype=dtype)
        w.fill(data)
        w.close()

        
def fermidistribution(energy, kt):
    #fermi level is fixed to zero
    return 1.0 / (1.0 + np.exp(energy / kt))

    
def zeroTFermi(energy):
    if np.real(energy) > 0:
        return 0.
    else:
        return 1.

        
def get_tri_type(mat):
    #mat is lower triangular or upper triangular matrix
    tol = 1e-10
    mat = abs(mat)
    dim = mat.shape[-1]
    sum = [0, 0]
    for i in range(dim):
        sum[0] += np.trace(mat, -j)
        sum[1] += np.trace(mat, j)
    diff = sum[0] - sum[1]
    if diff >= 0:
        ans = 'L'
    elif diff < 0:
        ans = 'U'
    if abs(diff) < tol:
        print('Warning: can not define the triangular matrix')
    return ans

    
def tri2full(M,UL='L'):
    """UP='L' => fill upper triangle from lower triangle
       such that M=M^d"""
    nbf = len(M)
    if UL=='L':
        for i in range(nbf-1):
            M[i,i:] = M[i:,i].conjugate()
    elif UL=='U':
        for i in range(nbf-1):
            M[i:,i] = M[i,i:].conjugate()

def dagger(matrix):
    return np.conj(matrix.T)

def get_matrix_index(ind1, ind2=None):
    if ind2 is None:
        dim1 = len(ind1)
        return np.resize(ind1, (dim1, dim1))
    else:
        dim1 = len(ind1)
        dim2 = len(ind2)
    return np.resize(ind1, (dim2, dim1)).T, np.resize(ind2, (dim1, dim2))
    
def aa1d(a, d=2):
    # array average in one dimension
    dim = a.shape
    b = [np.sum(np.take(a, [i], axis=d)) for i in range(dim[d])]
    b = np.array(b)
    b = (b * dim[d]) / np.product(dim)
    return b
    
def aa2d(a, d=0):
    # array average in two dimensions
    b = np.sum(a, axis=d) / a.shape[d]
    return b

def k2r_hs(h_skmm, s_kmm, ibzk_kc, weight_k, R_c=(0,0,0), magnet=None):
    phase_k = np.dot(2 * np.pi * ibzk_kc, R_c)
    c_k = np.exp(1.0j * phase_k) * weight_k
    c_k.shape = (len(ibzk_kc),1,1)

    if h_skmm is not None:
        nbf = h_skmm.shape[-1]
        nspins = len(h_skmm)
        h_smm = np.empty((nspins, nbf, nbf),complex)
        for s in range(nspins):
            h_smm[s] = np.sum((h_skmm[s] * c_k), axis=0)
    if s_kmm is not None:
        nbf = s_kmm.shape[-1]
        s_mm = np.empty((nbf, nbf),complex)
        s_mm[:] = np.sum((s_kmm * c_k), axis=0)
    #if magnet is not None:
    #    MM = magnet.trans_matrix(diag=True)
    #    assert np.sum(R_c) == R_c[2]
    #    MM = MM ** R_c[2]
    #    if h_skmm is not None:
    #        for s in range(nspins):
    #            h_smm[s] *= MM
    #    if s_kmm is not None:
    #        s_mm *= MM
    if h_skmm is not None and s_kmm is not None:
        return h_smm, s_mm
    elif h_skmm is None:
        return s_mm
    elif s_kmm is None:
        return h_smm

def r2k_hs(h_srmm, s_rmm, R_vector, kvector=(0,0,0), magnet=None):
    phase_k = np.dot(2 * np.pi * R_vector, kvector)
    c_k = np.exp(-1.0j * phase_k)
    c_k.shape = (len(R_vector), 1, 1)
   
    if h_srmm is not None:
        nbf = h_srmm.shape[-1]
        nspins = len(h_srmm)
        h_smm = np.empty((nspins, nbf, nbf), complex)
        for s in range(nspins):
            h_smm[s] = np.sum((h_srmm[s] * c_k), axis=0)
    if s_rmm is not None:
        nbf = s_rmm.shape[-1]
        s_mm = np.empty((nbf, nbf), complex)
        s_mm[:] = np.sum((s_rmm * c_k), axis=0)
#    if magnet is not None:
#        MM = magnet.trans_matrix(diag=True)
#       assert np.sum(R_vector) == R_vector[2]
#        MM = MM ** (-R_vector[2])
#       if h_srmm is not None:
#            for s in range(nspins):
#               h_smm[s] *= MM
#       if s_mm is not None:
#           s_mm *= MM
    if h_srmm is not None and s_rmm is not None:
        return h_smm, s_mm
    elif h_srmm is None:
        return s_mm
    elif s_rmm is None:
        return h_smm

def r2k2(s_rmm, R_vector, kvector=(0,0,0), symmetrize=True):
    phase_k = np.dot(2 * np.pi * R_vector, kvector)
    c_k = np.exp(-1.0j * phase_k)
    c_k.shape = (len(R_vector), 1, 1)
    nbf = s_rmm.shape[-1]
    s_mm = np.zeros((nbf, nbf), complex)
    for i in range(len(R_vector)):
        tmp = s_rmm[i,:,:]*c_k[i]
        if symmetrize and R_vector[i,:].any():
            s_mm = s_mm + tmp + dagger(tmp)
        else:
            s_mm = s_mm + tmp
    return s_mm

def collect_lead_mat(lead_hsd, lead_couple_hsd, s, pk, flag='S'):
    diag_h = []
    upc_h = []
    dwnc_h = []
    for i, hsd, c_hsd in zip(range(len(lead_hsd)), lead_hsd, lead_couple_hsd):
        if flag == 'S':
            band_mat, cp_mat = hsd.S[pk], c_hsd.S[pk]
        elif flag == 'H':
            band_mat, cp_mat = hsd.H[s][pk], c_hsd.H[s][pk]
        else:
            band_mat, cp_mat = hsd.D[s][pk], c_hsd.D[s][pk]
        diag_h.append(copy.deepcopy(band_mat))
        upc_h.append(cp_mat.recover('c'))
        dwnc_h.append(cp_mat.recover('n'))
    return diag_h, upc_h, dwnc_h
        
def get_hs(atoms):
    """Calculate the Hamiltonian and overlap matrix."""
    calc = atoms.calc
    wfs = calc.wfs
    wfs.gd.comm.broadcast(wfs.S_qMM, 0)
    Ef = calc.get_fermi_level()
    eigensolver = wfs.eigensolver
    ham = calc.hamiltonian
    S_qMM = wfs.S_qMM.copy()
    for S_MM in S_qMM:
        tri2full(S_MM)
    H_sqMM = np.empty((wfs.nspins,) + S_qMM.shape, wfs.dtype)
    for kpt in wfs.kpt_u:
        H_MM = eigensolver.calculate_hamiltonian_matrix(ham, wfs, kpt)
        tri2full(H_MM)
        H_MM *= Hartree
        #H_MM -= Ef * S_qMM[kpt.q]
        H_sqMM[kpt.s, kpt.q] = H_MM
    wfs.gd.comm.broadcast(H_sqMM, 0)
    return H_sqMM, S_qMM

def substract_pk(d, npk, ntk, kpts, k_mm, hors='s', position=[0, 0, 0], magnet=None):
    weight = np.array([1.0 / ntk] * ntk )
    if hors not in 'hs':
        raise KeyError('hors should be h or s!')
    if hors == 'h':
        dim = k_mm.shape[:]
        dim = (dim[0],) + (dim[1] // ntk,) + dim[2:]
        pk_mm = np.empty(dim, k_mm.dtype)
        dim = (dim[0],) + (ntk,) + dim[2:]
        tk_mm = np.empty(dim, k_mm.dtype)
    elif hors == 's':
        dim = k_mm.shape[:]
        dim = (dim[0] // ntk,) + dim[1:]
        pk_mm = np.empty(dim, k_mm.dtype)
        dim = (ntk,) + dim[1:]
        tk_mm = np.empty(dim, k_mm.dtype)

    tkpts = pick_out_tkpts(d, npk, ntk, kpts)
    for i in range(npk):
        n = i * ntk
        for j in range(ntk):
            if hors == 'h':
                tk_mm[:, j] = np.copy(k_mm[:, n + j])
            elif hors == 's':
                tk_mm[j] = np.copy(k_mm[n + j])
        if hors == 'h':
            pk_mm[:, i] = k2r_hs(tk_mm, None, tkpts, weight, position, magnet)
        elif hors == 's':
            pk_mm[i] = k2r_hs(None, tk_mm, tkpts, weight, position, magnet)
    return pk_mm

def pick_out_tkpts(d, npk, ntk, kpts):
    tkpts = np.zeros([ntk, 3])
    for i in range(ntk):
        tkpts[i, d] = kpts[i, d]
    return tkpts

def count_tkpts_num(d, kpts):
    tol = 1e-6
    tkpts = [kpts[0]]
    for kpt in kpts:
        flag = False
        for tkpt in tkpts:
            if abs(kpt[d] - tkpt[d]) < tol:
                flag = True
        if not flag:
            tkpts.append(kpt)
    return len(tkpts)
    
def dot(a, b, transa='n'):
    assert len(a.shape) == 2 and a.shape[1] == b.shape[0]
    dtype = complex
    if a.dtype == complex and b.dtype == complex:
        c = a
        d = b
    elif a.dtype == float and b.dtype == complex:
        c = np.array(a, complex)
        d = b
    elif a.dtype == complex and b.dtype == float:
        d = np.array(b, complex)
        c = a
    else:
        dtype = float
        c = a
        d = b
    e = np.zeros([c.shape[0], d.shape[1]], dtype)
    gemm(1.0, np.ascontiguousarray(d), np.ascontiguousarray(c), 0.0, e, transa)
    return e

def plot_diag(mtx, ind=1):
    import pylab
    dim = mtx.shape
    if len(dim) != 2:
        print('Warning! check the dimenstion of the matrix')
    if dim[0] != dim[1]:
        print('Warinng! check if the matrix is square')
    diag_element = np.diag(mtx)
    y_data = pick(diag_element, ind)
    x_data = range(len(y_data))
    pylab.plot(x_data, y_data,'b-o')
    pylab.show()

def get_atom_indices(subatoms, setups):
    basis_list = [setup.nao for setup in setups]
    index = []
    for j, lj  in zip(subatoms, range(len(subatoms))):
        begin = np.sum(np.array(basis_list[:j], int))
        for n in range(basis_list[j]):
            index.append(begin + n)
    return np.array(index, int)

def mp_distribution(e, kt, n=1):
    x = e / kt
    re = 0.5 * error_function(x)
    for i in range(n):
        re += coff_function(i + 1) * hermite_poly(2 * i + 1, x) * np.exp(-x**2)
    return re

def coff_function(n):
    return (-1)**n / (np.product(np.arange(1, n + 1)) * 4.** n * np.sqrt(np.pi))
    
def hermite_poly(n, x):
    if n == 0:
        return 1
    elif n == 1:
        return 2 * x
    else:
        return 2 * x * hermite_poly(n - 1, x) \
                                      - 2 * (n - 1) * hermite_poly(n - 2 , x)

def error_function(x):
    z = abs(x)
    t = 1. / (1. + 0.5*z)
    r = t * np.exp(-z*z-1.26551223+t*(1.00002368+t*(.37409196+
        t*(.09678418+t*(-.18628806+t*(.27886807+
        t*(-1.13520398+t*(1.48851587+t*(-.82215223+
        t*.17087277)))))))))
    if (x >= 0.):
        return r
    else:
        return 2. - r

def sum_by_unit(x, unit):
    dim = x.shape[0]
    dim1 = int(np.ceil(dim / unit))
    y = np.empty([dim1], dtype=x.dtype)
    for i in range(dim1 - 1):
        y[i] = np.sum(x[i * unit: (i + 1) * unit]) / unit
    y[0] = y[1]
    y[-1] = y[-2]
    return y

def diag_cell(cell):
    if len(cell.shape) == 2:
        cell = np.diag(cell)
    return cell
    
def get_pk_hsd(d, ntk, kpts, hl_skmm, sl_kmm, dl_skmm, txt=None,
                                                  dtype=complex, direction=0, magnet=None):
    npk = len(kpts) // ntk
    position = [0, 0, 0]
    hl_spkmm = substract_pk(d, npk, ntk, kpts, hl_skmm, hors='h', magnet=magnet)
    dl_spkmm = substract_pk(d, npk, ntk, kpts, dl_skmm, hors='h', magnet=magnet)
    sl_pkmm = substract_pk(d, npk, ntk, kpts, sl_kmm, hors='s', magnet=magnet)
    
    if direction==0:
        position[d] = 1
    else:
        position[d] = -1
    
    hl_spkcmm = substract_pk(d, npk, ntk, kpts, hl_skmm, 'h', position, magnet=magnet)
    dl_spkcmm = substract_pk(d, npk, ntk, kpts, dl_skmm, 'h', position, magnet=magnet)
    sl_pkcmm = substract_pk(d, npk, ntk, kpts, sl_kmm, 's', position, magnet=magnet)
    
    tol = 1e-6
    position[d] = 2.0
    s_test = substract_pk(d, npk, ntk, kpts, sl_kmm, 's', position, magnet=magnet)
    
    matmax = np.max(abs(s_test))
    if matmax > tol:
        if txt is not None:
            txt('Warning*: the principle layer should be larger, \
                                                      matmax=%f' % matmax)
        else:
            print('Warning*: the principle layer should be larger, \
                                                      matmax=%f' % matmax)
    if dtype == float:
        hl_spkmm = np.real(hl_spkmm).copy()
        sl_pkmm = np.real(sl_pkmm).copy()
        dl_spkmm = np.real(dl_spkmm).copy()
        hl_spkcmm = np.real(hl_spkcmm).copy()
        sl_pkcmm = np.real(sl_pkcmm).copy()
        dl_spkcmm = np.real(dl_spkcmm).copy()
    return hl_spkmm, sl_pkmm, dl_spkmm * ntk, hl_spkcmm, \
                                                    sl_pkcmm, dl_spkcmm * ntk
   
def get_lcao_density_matrix(calc):
    wfs = calc.wfs
    kpts = wfs.kd.ibzk_qc
    nq = len(kpts)
    my_ns = len(wfs.kpt_u) // nq
    nao = wfs.setups.nao
    
    # calculate_density_matrix involves gemm and doesn't work well with empty()
    d_skmm = np.zeros([my_ns, nq, nao, nao], wfs.dtype)
    for kpt in wfs.kpt_u:
        if my_ns == 1:
            wfs.calculate_density_matrix(kpt.f_n, kpt.C_nM, d_skmm[0, kpt.q])
        else:
            wfs.calculate_density_matrix(kpt.f_n, kpt.C_nM, d_skmm[kpt.s, kpt.q])
    return d_skmm

def collect_atomic_matrices(asp, setups, ns, comm, partition):
    all_asp = []
    for a, setup in enumerate(setups):
        sp = asp.get(a)
        if sp is None:
            ni = setup.ni
            sp = np.empty((ns, ni * (ni + 1) // 2))
        if comm.size > 1:
            comm.broadcast(sp, partition.rank_a[a])
        all_asp.append(sp)
    return all_asp

def distribute_atomic_matrices(all_asp, asp, setups):
    for a in range(len(setups)):
        if asp.get(a) is not None:
            asp[a] = all_asp[a]

def collect_and_distribute_atomic_matrices(D_ap, setups, setups0, partition, comm, keys):
    gD_ap = []
    D_ap0 = [None] * len(keys)
    for a, setup in enumerate(setups):
        if a not in keys:
            ni = setup.ni
            sp = np.empty((ni * (ni + 1) // 2))
        else:
            sp = D_ap[keys.index(a)]
        if comm.size > 1:
            comm.broadcast(sp, partition.rank_a[a])
        gD_ap.append(sp)
    for a in range(len(setups0)):
        if a in keys:
            D_ap0[keys.index(a)] = gD_ap[a]
    return D_ap0

def generate_selfenergy_database(atoms, ntk, filename, direction=0, kt=0.1,
                                 bias=[-3,3], depth=3, comm=None):
    from gpaw.transport.sparse_matrix import Banded_Sparse_HSD, CP_Sparse_HSD, Se_Sparse_Matrix
    from gpaw.transport.selfenergy import LeadSelfEnergy
    from gpaw.transport.contour import Contour
    hl_skmm, sl_kmm = get_hs(atoms)
    dl_skmm = get_lcao_density_matrix(atoms.calc)
    fermi = atoms.calc.get_fermi_level()
    wfs = atoms.calc.wfs
    hl_spkmm, sl_pkmm, dl_spkmm,  \
    hl_spkcmm, sl_pkcmm, dl_spkcmm = get_pk_hsd(2, ntk,
                                                wfs.kd.ibzk_qc,
                                                hl_skmm, sl_kmm, dl_skmm,
                                                None, wfs.dtype,
                                                direction=direction)
    my_npk = len(wfs.kd.ibzk_qc) / ntk
    my_nspins = len(wfs.kpt_u) / ( my_npk * ntk)
    
    lead_hsd = Banded_Sparse_HSD(wfs.dtype, my_nspins, my_npk)
    lead_couple_hsd = CP_Sparse_HSD(wfs.dtype, my_nspins, my_npk)
    for pk in range(my_npk):
        lead_hsd.reset(0, pk, sl_pkmm[pk], 'S', init=True)
        lead_couple_hsd.reset(0, pk, sl_pkcmm[pk], 'S', init=True)
        for s in range(my_nspins):
            lead_hsd.reset(s, pk, hl_spkmm[s, pk], 'H', init=True)
            lead_hsd.reset(s, pk, dl_spkmm[s, pk], 'D', init=True)
            lead_couple_hsd.reset(s, pk, hl_spkcmm[s, pk], 'H', init=True)
            lead_couple_hsd.reset(s, pk, dl_spkcmm[s, pk], 'D', init=True)
    lead_se = LeadSelfEnergy(lead_hsd, lead_couple_hsd)
    contour = Contour(kt, [fermi] * 2, bias, depth, comm=comm)
    path = contour.get_plot_path(ex=True)
    for nid, energy in zip(path.my_nids, path.my_energies):
        for kpt in wfs.kpt_u:
                if kpt.q % ntk == 0:
                    flag = str(kpt.k // ntk) + '_' + str(nid)
                    lead_se.s = kpt.s
                    lead_se.pk = kpt.q // ntk
                    data = lead_se(energy)
                    fd = open(flag, 'wb')
                    pickle.dump(data, fd, 2)
                    fd.close()

                    
def test_selfenergy_interpolation(atoms, ntk, filename, begin, end, base, scale, direction=0):
    from gpaw.transport.sparse_matrix import Banded_Sparse_HSD, CP_Sparse_HSD, Se_Sparse_Matrix
    from gpaw.transport.selfenergy import LeadSelfEnergy
    from gpaw.transport.contour import Contour
    hl_skmm, sl_kmm = get_hs(atoms)
    dl_skmm = get_lcao_density_matrix(atoms.calc)
    fermi = atoms.calc.get_fermi_level()
    wfs = atoms.calc.wfs
    hl_spkmm, sl_pkmm, dl_spkmm,  \
    hl_spkcmm, sl_pkcmm, dl_spkcmm = get_pk_hsd(2, ntk,
                                                wfs.kd.ibzk_qc,
                                                hl_skmm, sl_kmm, dl_skmm,
                                                None, wfs.dtype,
                                                direction=direction)
    my_npk = len(wfs.kd.ibzk_qc) / ntk
    my_nspins = len(wfs.kpt_u) / ( my_npk * ntk)
    
    lead_hsd = Banded_Sparse_HSD(wfs.dtype, my_nspins, my_npk)
    lead_couple_hsd = CP_Sparse_HSD(wfs.dtype, my_nspins, my_npk)
    for pk in range(my_npk):
        lead_hsd.reset(0, pk, sl_pkmm[pk], 'S', init=True)
        lead_couple_hsd.reset(0, pk, sl_pkcmm[pk], 'S', init=True)
        for s in range(my_nspins):
            lead_hsd.reset(s, pk, hl_spkmm[s, pk], 'H', init=True)
            lead_hsd.reset(s, pk, dl_spkmm[s, pk], 'D', init=True)
            lead_couple_hsd.reset(s, pk, hl_spkcmm[s, pk], 'H', init=True)
            lead_couple_hsd.reset(s, pk, dl_spkcmm[s, pk], 'D', init=True)
    lead_se = LeadSelfEnergy(lead_hsd, lead_couple_hsd)
    begin += fermi
    end += fermi
    
    ee = np.linspace(begin, end, base)
    cmp_ee = np.linspace(begin, end, base * scale)
  
    se = []
    cmp_se = []
    from scipy import interpolate

    for e in ee:
        se.append(lead_se(e).recover())
    se = np.array(se)
    ne, ny, nz= se.shape
    nie = len(cmp_ee)
    data = np.zeros([nie, ny, nz], se.dtype)
    for yy in range(ny):
        for zz in range(nz):
            ydata = se[:, yy, zz]
            f = interpolate.interp1d(ee, ydata)
            data[:, yy, zz] = f(cmp_ee)
    inter_se_linear = data
    
    for e in cmp_ee:
        cmp_se.append(lead_se(e).recover())
    
    fd = open(filename, 'wb')
    pickle.dump((cmp_se, inter_se_linear, ee, cmp_ee), fd, 2)
    fd.close()
    
    for i,e in enumerate(cmp_ee):
        print(e, np.max(abs(cmp_se[i] - inter_se_linear[i])), 'linear', np.max(abs(cmp_se[i])))


def path_selfenergy(atoms, ntk, filename, begin, end, num= 257, direction=0):
    from gpaw.transport.sparse_matrix import Banded_Sparse_HSD, CP_Sparse_HSD, Se_Sparse_Matrix
    from gpaw.transport.selfenergy import LeadSelfEnergy
    from gpaw.transport.contour import Contour
    hl_skmm, sl_kmm = get_hs(atoms)
    dl_skmm = get_lcao_density_matrix(atoms.calc)
    fermi = atoms.calc.get_fermi_level()
    wfs = atoms.calc.wfs
    hl_spkmm, sl_pkmm, dl_spkmm,  \
    hl_spkcmm, sl_pkcmm, dl_spkcmm = get_pk_hsd(2, ntk,
                                                wfs.kd.ibzk_qc,
                                                hl_skmm, sl_kmm, dl_skmm,
                                                None, wfs.dtype,
                                                direction=direction)
    my_npk = len(wfs.kd.ibzk_qc) / ntk
    my_nspins = len(wfs.kpt_u) / ( my_npk * ntk)
    
    lead_hsd = Banded_Sparse_HSD(wfs.dtype, my_nspins, my_npk)
    lead_couple_hsd = CP_Sparse_HSD(wfs.dtype, my_nspins, my_npk)
    for pk in range(my_npk):
        lead_hsd.reset(0, pk, sl_pkmm[pk], 'S', init=True)
        lead_couple_hsd.reset(0, pk, sl_pkcmm[pk], 'S', init=True)
        for s in range(my_nspins):
            lead_hsd.reset(s, pk, hl_spkmm[s, pk], 'H', init=True)
            lead_hsd.reset(s, pk, dl_spkmm[s, pk], 'D', init=True)
            lead_couple_hsd.reset(s, pk, hl_spkcmm[s, pk], 'H', init=True)
            lead_couple_hsd.reset(s, pk, dl_spkcmm[s, pk], 'D', init=True)
    lead_se = LeadSelfEnergy(lead_hsd, lead_couple_hsd)
    begin += fermi
    end += fermi
    
    ee = np.linspace(begin, end, num)
    se = []

    for e in ee:
        se.append(lead_se(e).recover())
    se = np.array(se)
    
    fd = open(filename + '_' + str(world.rank), 'wb')
    pickle.dump((se, ee), fd, 2)
    fd.close()

def sort_atoms(atoms):
    pos = atoms.positions.copy()
    ind2 = fuzzy_sort(pos[:,2])
    ind1 = fuzzy_sort(pos[:,1])
    ind0 = fuzzy_sort(pos[:,0])
    tmp = ind2 * 1e12 + ind1 * 1e6 + ind0
    indices = np.argsort(tmp)
    atoms.positions = atoms.positions[indices]
    atoms.numbers = atoms.numbers[indices]
    if 'magmoms' in atoms.arrays:
        atoms.arrays['magmoms'] = atoms.arrays['magmoms'][indices]

def fuzzy_sort(seq0, tol=1e-6):
    seq = seq0.copy()
    ind0 = np.zeros_like(seq)
    ind1 = []
    n = 0
    while len(ind1) < len(seq):
        ind = []
        am = np.argmin(seq)
        tmp = seq - seq[am]
        for j, i in enumerate(tmp):
            if abs(i) < tol:
                ind.append(j)
        seq[ind] = 1e19
        ind0[ind] = n
        ind1 += ind
        n += 1
    return ind0

def cubicing(atoms):
    cell = atoms._cell
    positions = atoms.positions
    print('cubicing only ok to [a,0,0][a/2, b, 0],[0,0,c] type ')
    tol = 1e-6
    if abs(cell[1,0]*2 - cell[0,0]) < tol:
        print('ok, possible to get a cubic structure')
        natoms = len(positions)
        new_pos = np.empty([natoms * 2, 3])
        for pos, i in zip(positions, range(natoms)):
            new_pos[i] = pos
            new_pos[i + natoms] = pos + cell[1]
        atoms += atoms
        atoms.positions = new_pos
        sort_atoms(atoms)
        cell[1, 0] = 0
        cell[1, 1] *= 2
        atoms.set_cell(cell)
    return atoms

class P_info:
    def __init__(self):
        self.x = 0
        self.y = 0
        self.z = 0
        self.Pxsign = 1
        self.Pysign = 1
        self.Pzsign = 1
        self.N = 0
class D_info:
    def __init__(self):
        self.xy = 0
        self.xz = 0
        self.yz = 0
        self.x2y2 = 0
        self.z2r2 = 0
        self.N = 0

def egodic(nums):
    if len(nums)==1:
        return np.array(nums)
    else:
        rows = np.product(np.arange(1, len(nums) + 1))
        cols = len(nums)
        all = np.zeros([rows, cols])
        
        for i, n in enumerate(nums):
            subrows = np.product(np.arange(1, len(nums)))
            all[i*subrows: (i+1)*subrows, 0] = n
            left_nums = nums[:]
            left_nums.remove(n)
            all[i*subrows: (i+1)*subrows, 1:] = egodic(left_nums)
        return all
    
#PPP = egodic(range(3))

def PutP(index, X, P, T):
    if P.N == 2:
        P.x = index
    if P.N == 0:
        P.y = index
    if P.N == 1:
        P.z = index
    P.N += 1
    
    if P.N == 3:
        bs = np.array([P.x, P.y, P.z])
        #c = np.array([P.Pxsign, P.Pysign, P.Pzsign])
        #c = np.resize(c, [3, 3])
        #cf = c / c.T
        #ind = np.resize(bs, [3, 3])
        ind = get_matrix_index(bs)
        T[ind.T, ind] = X
        #T[ind.T, ind] = X * cf
        P.__init__()

#DDD = egodic(range(5))

def PutD(index, X, D, T):
    if D.N == 0:
        D.xy = index
    if D.N == 3:
        D.xz = index
    if D.N == 1:
        D.yz = index
    if D.N == 4:
        D.x2y2 = index
    if D.N == 2:
        D.z2r2 = index
        
    D.N += 1
    if D.N == 5:
        sqrt = np.sqrt
        Dxy = np.array([[0, 1, 0],
                        [1, 0, 0],
                        [0, 0, 0]])
        D2xy = np.dot(X, Dxy)
        D2xy = np.dot(D2xy, X.T)
        
        Dxz = np.array([[0, 0, 1],
                        [0, 0, 0],
                        [1, 0, 0]])
        D2xz = np.dot(X, Dxz)
        D2xz = np.dot(D2xz, X.T)
        
        Dyz = np.array([[0, 0, 0],
                        [0, 0, 1],
                        [0, 1, 0]])
        D2yz = np.dot(X, Dyz)
        D2yz = np.dot(D2yz, X.T)

        Dx2y2 = np.array([[1, 0 , 0],
                          [0, -1, 0],
                          [0, 0,  0]])
        D2x2y2 = np.dot(X, Dx2y2)
        D2x2y2 = np.dot(D2x2y2, X.T)
        
        Dz2r2 = np.array([[-1, 0, 0],
                          [0, -1, 0],
                          [0,  0, 2]]) / sqrt(3)
        D2z2r2 = np.dot(X, Dz2r2)
        D2z2r2 = np.dot(D2z2r2, X.T)
        
        T[D.xy, D.xy] = D2xy[0, 1]
        T[D.xz, D.xy] = D2xy[0, 2]
        T[D.yz, D.xy] = D2xy[1, 2]
        T[D.x2y2, D.xy] = (D2xy[0, 0] - D2xy[1, 1]) / 2
        T[D.z2r2, D.xy] = sqrt(3) / 2 * D2xy[2, 2]

        T[D.xy, D.xz] = D2xz[0, 1]
        T[D.xz, D.xz] = D2xz[0, 2]
        T[D.yz, D.xz] = D2xz[1, 2]
        T[D.x2y2, D.xz] = (D2xz[0, 0] - D2xz[1, 1]) / 2
        T[D.z2r2, D.xz] = sqrt(3) / 2 * D2xz[2,2];

        T[D.xy , D.yz] = D2yz[0, 1]
        T[D.xz , D.yz] = D2yz[0, 2]
        T[D.yz , D.yz] = D2yz[1, 2]
        T[D.x2y2, D.yz] = (D2yz[0, 0] - D2yz[1, 1]) / 2
        T[D.z2r2, D.yz] = sqrt(3) / 2 * D2yz[2, 2]

        T[D.xy , D.x2y2] = D2x2y2[0, 1]
        T[D.xz , D.x2y2] = D2x2y2[0, 2]
        T[D.yz , D.x2y2] = D2x2y2[1, 2]
        T[D.x2y2, D.x2y2] = (D2x2y2[0, 0] - D2x2y2[1, 1]) / 2
        T[D.z2r2, D.x2y2] = sqrt(3) / 2 * D2x2y2[2, 2]

        T[D.xy, D.z2r2] = D2z2r2[0, 1]
        T[D.xz, D.z2r2] = D2z2r2[0, 2]
        T[D.yz, D.z2r2] = D2z2r2[1, 2]
        T[D.x2y2, D.z2r2] = (D2z2r2[0, 0] - D2z2r2[1, 1]) / 2
        T[D.z2r2, D.z2r2] = sqrt(3) / 2 * D2z2r2[2, 2]
        
        D.__init__()
        
def orbital_matrix_rotate_transformation(X, orbital_indices):
    nb = orbital_indices.shape[0]
    assert len(X) == 3
    T = np.zeros([nb, nb])
    P = P_info()
    D = D_info()
    for i in range(nb):
        if orbital_indices[i, 1] == 0:
            T[i, i] = 1
        elif orbital_indices[i, 1] == 1:
            PutP(i, X, P, T)
        elif orbital_indices[i, 1] == 2:
            PutD(i, X, D, T)
        else:
            raise NotImplementError('undown shell name')
    return T

def normalize(r):
    return r/np.sqrt(np.sum(r*r))

def vector_to_paramid(r):
    r = normalize(r)
    x, y, z = r
    if z!=0:
        a1, b1, c1 = 0, 1, -y/z
        a2, b2, c2 = 1, -x*y/(y**2 + z**2), -x*z/(y**2 + z**2)
    elif y!=0:
        a1, b1, c1 = 1, -x/y, 0
        a2, b2, c2 = -x*z/(y**2 + x**2), -y*z/(y**2 + x**2), 1
    elif x!=0:
        a1, b1, c1 = -z/x, 0, 1
        a2, b2, c2 = -y*z/(x**2 + z**2), 1, -x*y/(x**2 + z**2)
    else:
        raise RuntimeError('The input vector is zero!')
    r1 = np.array([a1, b1, c1])
    r2 = np.array([a2, b2, c2])
    r1 = normalize(r1)
    r2 = normalize(r2)
    R1 = r + r1
    R2 = r - r1 / 2. + r2 / 2.
    R3 = r - r1 / 2. - r2 / 2.
    return np.array([R1, R2, R3])
  
def transform_3d(rs1, rs2):
    assert rs1.shape == rs2.shape
    if rs1.shape[0] == 2:
        r1 = rs1[1] - rs1[0]
        r2 = rs2[1] - rs2[0]
        RS1 = vector_to_paramid(r1)
        RS2 = vector_to_paramid(r2)
    elif rs1.shape[0] == 4:
        RS1 = rs1[1:] - rs1[0]
        RS2 = rs2[1:] - rs2[0]
    else:
        raise RuntimeError('Transform atoms indices wrong!')
    X = np.dot(RS2.T, np.linalg.inv(RS1.T))
    return X

def interpolate_2d(mat):
    from gpaw.grid_descriptor import GridDescriptor
    from gpaw.transformers import Transformer
    nn = 10
    N_c = np.zeros([3], dtype=int)
    N_c[1:] = mat.shape[:2]
    N_c[0] = nn
    bmat = np.resize(mat, N_c)
    gd = GridDescriptor(N_c, N_c)
    finegd = GridDescriptor(N_c * 2, N_c)
    interpolator = Transformer(gd, finegd, 3)
    fine_bmat = finegd.zeros()
    interpolator.apply(bmat, fine_bmat)
    return fine_bmat[0]
    
def interpolate_array(array, gd, h, di='+'):
    try:
        from scipy import interpolate
        ip = True
    except ImportError:
        ip = False
    if not ip:
        return array

    dim = len(array.shape)
    assert dim == 3 or dim == 4
    spin_relate = dim == 4
    if h <= gd.h_cv[2, 2]:
        if di == '+':
            x = np.arange(gd.N_c[2]) * gd.h_cv[2, 2]
            xnew = np.arange(gd.N_c[2]) * h
        else:
            x = np.arange(-gd.N_c[2], 0) * gd.h_cv[2, 2]
            xnew = np.arange(-gd.N_c[2], 0) * h
    else:
        if di == '+':
            x = np.arange(gd.N_c[2] * 2) * gd.h_cv[2, 2]
            xnew = np.arange(gd.N_c[2]) * h
        else:
            x = np.arange(-gd.N_c[2] * 2, 0) * gd.h_cv[2, 2]
            xnew = np.arange(-gd.N_c[2], 0) * h
        
    if spin_relate:
        ns, nx, ny, nz = array.shape
        array.shape = (ns * nx * ny, nz)
        new_array = gd.zeros(ns, global_array=True)
        new_array.shape = (ns * nx * ny, nz)
    else:
        nx, ny, nz = array.shape
        array.shape = (nx * ny, nz)
        new_array = gd.zeros(global_array=True)
        new_array.shape = (nx * ny, nz)
      
    if h > gd.h_cv[2, 2]:
        array = np.append(array, array, 1)
        
    for i, line in enumerate(array):
        tck = interpolate.splrep(x, line, s=0)
        new_array[i] = interpolate.splev(xnew, tck, der=0)
    
    if spin_relate:
        #array.shape = (ns, nx, ny, nz)
        new_array.shape = (ns, nx, ny, nz)
    else:
        #array.shape = (nx, ny, nz)
        new_array.shape = (nx, ny, nz)
    
    return new_array
        
def eig_states_norm(orbital, s_mm):
    #normalize orbital to satisfy orbital.T.conj()*SM*orbital=unit
    norm_error = 1e-10
    ortho_error = 1e-8
    nstates = orbital.shape[1]
    d_mm = np.dot(orbital.T.conj(), s_mm)
    d_mm = np.dot(d_mm, orbital)
    for i in range(1, nstates):
        for j in range(i):
            if abs(d_mm[j ,i]) > ortho_error:
                orbital[:, i] -= orbital[:, j] * d_mm[j, i] / d_mm[j ,j]
                d_mm = np.dot(orbital.T.conj(), s_mm)
                d_mm = np.dot(d_mm, orbital)
    for i in range(nstates):
        orbital[:, i] /= np.sqrt(d_mm[i, i])

    if orbital.shape[-1] == 0:
        error = 0
    else:
        error = np.max(np.dot(np.dot(orbital.T.conj(), s_mm), orbital) -
                   np.eye(nstates)) / nstates
  
    if  abs(error) > norm_error:
        print('Warning! Normalization error %f' % error)
    return orbital

def shtm(l):
    #Spherical Harmonics transformation matrix, from the complex to real
    #The harmonics should be arranged in sequence -m, -m+1 ... m-1, m
    n = 2 * l + 1
    mtx = np.zeros([n,n], complex)
    for i in range(n):
        if i < l:
            #mtx[i, i] = 1.j*(-1.)**(l - i) / np.sqrt(2)
            #mtx[i, i + 2*(l-i)] = -1.j / np.sqrt(2)
            # The change is due to Condon-Shortley phase
            mtx[i, i] = 1.j / np.sqrt(2)
            mtx[i, i + 2*(l-i)] = -1.j*(-1.)**(l - i) / np.sqrt(2)
        elif i == l:
            mtx[i, i] = 1.
        else:
            #mtx[i, i] = 1./np.sqrt(2)
            #mtx[i, i + 2*(l-i)] = (-1.)**(i - l) / np.sqrt(2)
            mtx[i, i] = (-1.)**(i - l) /np.sqrt(2)
            mtx[i, i + 2*(l-i)] = 1. / np.sqrt(2)
    return mtx.T

def construct_spherical_transformation_matrix(l_list):
    #construct a transformation matrix from complex harmonics to real for
    #lcao basis
    nao = np.sum([2 * l + 1 for l in l_list])
    mtx = np.zeros([nao, nao], complex)
    start = 0
    for l in l_list:
        n = 2 * l + 1
        mtx[start:start+n, start:start+n] = shtm(l)
        start += n
    return mtx

def aml(ss, l, direction):
    #calculat angular momentum matrix(complex spherical harmonics)
    #elements based on the overlap
    amss = np.zeros(ss.shape, complex)
    n = 2 * l + 1
    for i in range(n):
        m = i - l
        # x direction=0, lx=(l+ + l-) /2
        if direction == 0:
            a1 = 0.5 * np.sqrt(l*(l+1.)-m*(m+1.))
            a2 = 0.5 * np.sqrt(l*(l+1.)-m*(m-1.))
        # y direction=1, ly=(l+ - l-) /2i
        if direction == 1:
            a1 = -0.5 * 1.j * np.sqrt(l*(l+1.)-m*(m+1.))
            a2 = 0.5 * 1.j * np.sqrt(l*(l+1.)-m*(m-1.))
        if direction == 0 or direction == 1:
            if m + 1 <= l:
                amss[i] += a1 * ss[i + 1]
            if m - 1 >= -l:
                amss[i] += a2 * ss[i - 1]
        elif direction == 2: #z direction=2
            amss[i] = m * ss[i]
        else:
            raise RuntimeError('unknown direction %d' % direction)
    return amss

def angular_momentum_slice(overlap_slice, l, direction):
    #given a overlap matrix slice <l m| l_i,m> for a fixed l_i
    #return the angular momentum matrix slice <l m| l |l_i, m>
    #the slice has a shape (nao, 2*l_i + 1)
    #the magnetic number m is in the increasing sequence
    nao, n = overlap_slice.shape
    am_slice = np.zeros([nao, n], complex)
    for i in range(nao):
        ss = overlap_slice[i]
        am_slice[i] = aml(ss, l, direction)
    return am_slice

def cut_grids_side(array, gd, gd0):
    #abstract the grid value from a including-buffer-layer calculation
    #the vacuum buffer layer is fixed on the right side
    #Assume the buffer system has the same domain spliting with the original one
    from scipy import interpolate
    global_array = gd.collect(array)
    nx, ny, nz = gd.N_c
    if gd.comm.rank == 0:
        global_array.shape = (nx * ny, nz)
    new_array = gd0.zeros()
    global_new_array = gd0.collect(new_array)
    x = np.arange(gd.N_c[2]) * gd.h_cv[2, 2]
    xnew = np.arange(gd0.N_c[2]) * gd0.h_cv[2, 2]
    nz0 = gd0.N_c[2]
    if gd0.comm.rank == 0:
        global_new_array.shape = (nx * ny, nz0)
        for i, line in enumerate(global_array):
            tck = interpolate.splrep(x, line, s=0)
            global_new_array[i] = interpolate.splev(xnew, tck, der=0)
        global_new_array.shape = (nx, ny, nz0)
    gd0.distribute(global_new_array, new_array)
    return new_array


def save_bias_data_file(Lead1, Lead2, Device):
    ham = Device.calc.hamiltonian
    density = Device.calc.density
    hamL = Lead1.calc.hamiltonian
    hamR = Lead2.calc.hamiltonian
    Ef = Device.calc.get_fermi_level()
    Ef_L = Lead1.calc.get_fermi_level()
    Ef_R = Lead2.calc.get_fermi_level()
    vt_sG = ham.gd.collect(ham.vt_sG)
    vt_sG_L = hamL.gd.collect(hamL.vt_sG)
    vt_sG_R = hamR.gd.collect(hamR.vt_sG)
    dH_asp = collect_atomic_matrices(ham.dH_asp, ham.setups,
                                     ham.nspins, ham.gd.comm,
                                     density.atom_partition)
    if world.rank == 0:
        vt_sG += (Ef_L-Ef) / Hartree
        vt_sG_R += (Ef_L - Ef_R) / Hartree
        vt_sG=np.append(vt_sG_L, vt_sG,axis=3)
        vt_sG=np.append(vt_sG,vt_sG_R,axis=3)
        pickle.dump(([0.0,0,0], vt_sG, dH_asp), open('bias_data1', 'wb'), 2)

def find(condition, flag=0):
    if flag == 1: # return an int
        return np.int(np.nonzero(condition)[0])
    else: # return an array
        return np.nonzero(condition)[0]
   
def gather_ndarray_list(data, comm):
    #data is a numpy array, maybe has different shape in different cpus
    #this function gather them to the all_data in master, all_data is
    # a list with the lenth world.size, all_data[i] = data {on i}
    all_data = []
    dim = len(data.shape)
    shape_array = np.zeros([comm.size, dim], int)
    shape_array[comm.rank] = data.shape
    comm.sum(shape_array)
    
    if comm.rank == 0:
        all_data.append(data)
        for i in range(1, comm.size):
            tmp = np.zeros(shape_array[i], dtype=data.dtype)
            comm.receive(tmp, i, 546)
            all_data.append(tmp[:])
    else:
        comm.ssend(data, 0, 546)
    return all_data
        
def gather_ndarray_dict(data, comm, broadcast=False):
    #data is dict of a numpy array, maybe has different shape in different cpus
    #this function gather them to the all_data in master, all_data is
    # a dict with the lenth world.size
    all_data = {}
    data_len = np.zeros([comm.size], int)
    data_len[comm.rank] = len(data)
    comm.sum(data_len)
    
    info = np.zeros([np.sum(data_len), 3], int)
    dtypes = [int, float, complex]
    for i, name in enumerate(data):
        base = np.sum(data_len[:comm.rank])
        info[base + i, 0] = len(data[name].shape)
        info[base + i, 1] = comm.rank
        info[base + i, 2] = dtypes.index(data[name].dtype)
    comm.sum(info)
    
    if comm.rank == 0:
        for name in data:
            all_data[name] = data[name]
 
        for i in range(1, comm.size):
            base = np.sum(data_len[:i])
            for j in range(data_len[i]):
                shape = np.zeros([info[base + j, 0]], int)
                dtype = dtypes[info[base + j, 2]]
                name = receive_string(i, comm)
                comm.receive(shape, i, 123)
                tmp = np.zeros(shape, dtype)
                comm.receive(tmp, i, 546)
                all_data[name] = tmp

    else:
        for name in data:
            send_string(name, 0, comm)
            shape = np.array(data[name].shape, int)
            comm.ssend(shape, 0, 123)
            comm.ssend(data[name], 0, 546)
    
    if broadcast:
        num = np.zeros([1], int)
        if comm.rank == 0:
            num[0] = len(all_data)
            comm.broadcast(num, 0)
            for name in all_data:
                broadcast_string(name, 0, comm)
                shape = np.array(all_data[name].shape, int)
                comm.broadcast(shape, 0)
                comm.broadcast(all_data[name], 0)
        else:
            comm.broadcast(num, 0)
            for i in range(num):
                name = broadcast_string(None, 0, comm)
                shape = np.zeros([info[i, 0]], int)
                dtype = dtypes[info[i, 2]]
                comm.broadcast(shape, 0)
                tmp = np.zeros(shape, dtype)
                comm.broadcast(tmp, 0)
                all_data[name] = tmp
    return all_data
    
    



from __future__ import print_function
import sys
import numpy as np

from ase.structure import molecule
from ase.parallel import parprint

from gpaw import GPAW, PW
from gpaw.analyse.multipole import Multipole
from gpaw.cluster import Cluster
from gpaw.test import equal

h = 0.3

s = Cluster(molecule('H2O'))
s.minimal_box(3., h)

gpwname = 'H2O_h' + str(h) + '.gpw'
try:
    # XXX check why this fails in parallel
    calc = GPAW(gpwname + 'failsinparallel', txt=None)
    atoms = calc.get_atoms()
    calc.density.set_positions(atoms.get_scaled_positions() % 1.0)
    calc.density.interpolate_pseudo_density()
    calc.density.calculate_pseudo_charge()
except IOError:
    calc = GPAW(h=h, charge=0, txt=None)
    calc.calculate(s)
    calc.write(gpwname)

dipole_c = calc.get_dipole_moment()
parprint('Dipole', dipole_c) 

center = np.array([1,1,1]) * 50.
mp = Multipole(center, calc, lmax=2)
q_L = mp.expand(-calc.density.rhot_g)
parprint('Multipole', q_L)

# The dipole moment is independent of the center
equal(dipole_c[2], q_L[2], 1e-10)

mp.to_file(calc, mode='w')
